from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.msft import azure_rule_context, azure_success


@panther_managed
class AzureAuditPolicyChanged(Rule):
    id = "Azure.Audit.PolicyChanged-prototype"
    display_name = "Azure Policy Changed"
    log_types = [LogType.AZURE_AUDIT]
    default_severity = Severity.LOW
    dedup_period_minutes = 10
    default_description = "This detection looks for policy changes in AuditLogs\n"
    reports = {"MITRE ATT&CK": ["TA0005:T1526"]}
    default_runbook = "Verify if the change was authorized and review the modifications. If unauthorized, revert the policy, notify relevant teams, and investigate the user actions.\n"
    default_reference = "https://learn.microsoft.com/en-us/entra/identity/authentication/overview-authentication"
    summary_attributes = [
        "properties:ServicePrincipalName",
        "properties:UserPrincipalName",
        "properties:initiatedBy:user:ipAddress",
    ]
    POLICY_OPERATION = "policy"
    IGNORE_ACTIONS = ["Add", "Added"]

    def rule(self, event):
        operation = event.get("operationName", default="")
        if not azure_success(event) or not operation.endswith(self.POLICY_OPERATION):
            return False
        # Ignore added policies
        if any(operation.startswith(ignore) for ignore in self.IGNORE_ACTIONS):
            return False
        return True

    def title(self, event):
        operation_name = event.get("operationName", default="<UNKNOWN OPERATION>")
        actor_name = event.deep_get("properties", "initiatedBy", "user", "userPrincipalName", default="<UNKNOWN ACTOR>")
        policy = event.deep_walk("properties", "targetResources", "displayName", default="<UNKNOWN POLICY>")
        return f"{operation_name} by {actor_name} on the policy {policy}"

    def alert_context(self, event):
        return azure_rule_context(event)

    tests = [
        RuleTest(
            name="Policy Changed",
            expected_result=True,
            log={
                "time": "2024-12-10T02:22:58.7270280Z",
                "resourceId": "/tenants/123145/providers/Microsoft.aadiam",
                "operationName": "Delete conditional access policy",
                "operationVersion": "1.0",
                "category": "AuditLogs",
                "tenantId": "12341234",
                "resultSignature": "None",
                "durationMs": 0,
                "callerIpAddress": "1.2.3.4",
                "correlationId": "1324515",
                "Level": 4,
                "properties": {
                    "tenantId": "132455112",
                    "resultType": "",
                    "resultDescription": "",
                    "operationName": "Delete conditional access policy",
                    "identity": "",
                    "tenantGeo": "NA",
                    "id": "IPCGraph_af23466234",
                    "category": "Policy",
                    "correlationId": "23456234",
                    "result": "success",
                    "resultReason": None,
                    "activityDisplayName": "Delete conditional access policy",
                    "activityDateTime": "2024-11-27T02:22:58.727028+00:00",
                    "loggedByService": "Conditional Access",
                    "operationType": "Delete",
                    "userAgent": None,
                    "initiatedBy": {
                        "user": {
                            "id": "234526234",
                            "displayName": None,
                            "userPrincipalName": "Kratos@mtolympus.com",
                            "ipAddress": "1.2.3.4",
                            "roles": [],
                        },
                    },
                    "targetResources": [
                        {
                            "id": "5e3cb481-2814-4295-b4cc-2440a6d66c86",
                            "displayName": "Outside MFA",
                            "type": "Policy",
                            "modifiedProperties": [
                                {
                                    "displayName": "ConditionalAccessPolicy",
                                    "oldValue": '{"id":"5e3cb481-2814-4295-b4cc-2440a6d66c86","displayName":"Outside MFA","createdDateTime":"2024-11-27T02:22:19.8926587+00:00","state":"enabled","conditions":{"applications":{"includeApplications":["None"],"excludeApplications":[],"includeUserActions":[],"includeAuthenticationContextClassReferences":[],"applicationFilter":null},"users":{"includeUsers":[],"excludeUsers":[],"includeGroups":[],"excludeGroups":[],"includeRoles":[],"excludeRoles":[],"includeGuestsOrExternalUsers":{"guestOrExternalUserTypes":63,"externalTenants":{}}},"userRiskLevels":[],"signInRiskLevels":[],"clientAppTypes":["all"],"servicePrincipalRiskLevels":[]},"grantControls":{"operator":"OR","builtInControls":["mfa"],"customAuthenticationFactors":[],"termsOfUse":[]}}',
                                    "newValue": None,
                                },
                            ],
                            "administrativeUnits": [],
                        },
                    ],
                    "additionalDetails": [{"key": "Category", "value": "Conditional Access"}],
                },
            },
        ),
        RuleTest(
            name="Policy Updated",
            expected_result=True,
            log={
                "time": "2024-11-21T16:47:21.6424070Z",
                "resourceId": "/tenants/1234155/providers/Microsoft.aadiam",
                "operationName": "Update policy",
                "operationVersion": "1.0",
                "category": "AuditLogs",
                "tenantId": "12341235",
                "resultSignature": "None",
                "durationMs": 0,
                "callerIpAddress": "1.2.3.4",
                "correlationId": "124412123",
                "Level": 4,
                "properties": {
                    "tenantId": "123515-5-1235",
                    "resultType": "",
                    "resultDescription": "",
                    "operationName": "Update policy",
                    "identity": "",
                    "tenantGeo": "NA",
                    "id": "Directory_12351123",
                    "category": "Policy",
                    "correlationId": "12341513",
                    "result": "success",
                    "resultReason": "",
                    "activityDisplayName": "Delete policy",
                    "activityDateTime": "2024-11-21T16:47:21.642407+00:00",
                    "loggedByService": "Core Directory",
                    "operationType": "Delete",
                    "userAgent": None,
                    "initiatedBy": {
                        "user": {
                            "id": "12345123",
                            "displayName": None,
                            "userPrincipalName": "Kratos@onmicrosoft.com",
                            "ipAddress": "1.2.3.4",
                            "roles": [],
                        },
                    },
                    "targetResources": [
                        {
                            "id": "123451516",
                            "displayName": "Require multifactor authentication for all users",
                            "type": "Policy",
                            "modifiedProperties": [],
                            "administrativeUnits": [],
                        },
                    ],
                    "additionalDetails": [{"key": "User-Agent", "value": "Microsoft Azure Graph Client Library 1.0"}],
                },
            },
        ),
        RuleTest(
            name="Policy Added",
            expected_result=False,
            log={
                "time": "2024-11-21T16:47:21.6424070Z",
                "resourceId": "/tenants/1234155/providers/Microsoft.aadiam",
                "operationName": "Added policy",
                "operationVersion": "1.0",
                "category": "AuditLogs",
                "tenantId": "12341235",
                "resultSignature": "None",
                "durationMs": 0,
                "ipAddress": "1.2.3.4",
                "correlationId": "124412123",
                "Level": 4,
                "properties": {
                    "tenantId": "123515-5-1235",
                    "resultType": "",
                    "resultDescription": "",
                    "operationName": "Added policy",
                    "identity": "",
                    "tenantGeo": "NA",
                    "id": "Directory_12351123",
                    "category": "Policy",
                    "correlationId": "12341513",
                    "result": "success",
                    "resultReason": "",
                    "activityDisplayName": "Added policy",
                    "activityDateTime": "2024-11-21T16:47:21.642407+00:00",
                    "loggedByService": "Core Directory",
                    "operationType": "Delete",
                    "userAgent": None,
                    "initiatedBy": {
                        "user": {
                            "id": "12345123",
                            "displayName": None,
                            "userPrincipalName": "Kratos@onmicrosoft.com",
                            "ipAddress": "1.2.3.4",
                            "roles": [],
                        },
                    },
                    "targetResources": [
                        {
                            "id": "123451516",
                            "displayName": "Require multifactor authentication for all users",
                            "type": "Policy",
                            "modifiedProperties": [],
                            "administrativeUnits": [],
                        },
                    ],
                    "additionalDetails": [{"key": "User-Agent", "value": "Microsoft Azure Graph Client Library 1.0"}],
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed


@panther_managed
class SnowflakeStreamExternalShares(Rule):
    id = "Snowflake.Stream.ExternalShares-prototype"
    display_name = "Snowflake External Data Share"
    log_types = [LogType.SNOWFLAKE_DATA_TRANSFER_HISTORY]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0010:T1537"]}
    default_description = (
        "Detect when an external share has been initiated from one source cloud to another target cloud."
    )
    default_runbook = "Determine if this occurred as a result of a valid business request."
    tags = ["Configuration Required", "Snowflake", "[MITRE] Exfiltration", "[MITRE] Transfer Data to Cloud Account"]
    # CONFIGURATION REQUIRED
    #   Be sure to add code to exclude any transfers from acounts designed to host data shares. Either
    #   add those account names to the set below, or add a rule filter to exclude events with those
    #   account names.
    # Add account names here
    DATA_SHARE_HOSTING_ACCOUNTS = {}

    def rule(self, event):
        return all(
            [
                event.get("ACCOUNT_NAME") not in self.get_data_share_hosting_accounts(),
                event.get("SOURCE_CLOUD"),
                event.get("TARGET_CLOUD"),
                event.get("BYTES_TRANSFERRED", 0) > 0,
            ],
        )

    def title(self, event):
        return f"{event.get('ORGANIZATION_NAME', '<UNKNOWN ORGANIZATION>')}: A data export has been initiated from source cloud {event.get('SOURCE_CLOUD', '<UNKNOWN SOURCE CLOUD>')} in source region {event.get('SOURCE_REGION', '<UNKNOWN SOURCE REGION>')} to target cloud {event.get('TARGET_CLOUD', '<UNKNOWN TARGET CLOUD>')} in target region {event.get('TARGET_REGION', '<UNKNOWN TARGET REGION>')} with transfer type {event.get('TRANSFER_TYPE', '<UNKNOWN TRANSFER TYPE>')} for {event.get('BYTES_TRANSFERRED', '<UNKNOWN VOLUME>')} bytes"

    def get_data_share_hosting_accounts(self):
        """Getter function. Used so we can mock during unit tests."""
        return self.DATA_SHARE_HOSTING_ACCOUNTS

    tests = [
        RuleTest(
            name="Allowed Share",
            expected_result=False,
            mocks=[
                RuleMock(
                    object_name="get_data_share_hosting_accounts",
                    return_value="{DP_EUROPE}, {DP_ASIA}, {DP_AMERICA}",
                ),
            ],
            log={
                "ORGANIZATION_NAME": "DAILY_PLANET",
                "ACCOUNT_NAME": "DP_EUROPE",
                "REGION": "US-EAST-2",
                "SOURCE_CLOUD": "AWS",
                "SOURCE_REGION": "US-EAST-2",
                "TARGET_CLOUD": "AWS",
                "TARGET_REGION": "EU-WEST-1",
                "BYTES_TRANSFERRED": 61235879,
                "TRANSFER_TYPE": "COPY",
            },
        ),
        RuleTest(
            name="Disallowed Share",
            expected_result=True,
            mocks=[
                RuleMock(
                    object_name="get_data_share_hosting_accounts",
                    return_value="{DP_EUROPE}, {DP_ASIA}, {DP_AMERICA}",
                ),
            ],
            log={
                "ORGANIZATION_NAME": "LEXCORP",
                "ACCOUNT_NAME": "LEX_SECRET_ACCOUNT",
                "REGION": "US-EAST-2",
                "SOURCE_CLOUD": "AWS",
                "SOURCE_REGION": "US-EAST-2",
                "TARGET_CLOUD": "AWS",
                "TARGET_REGION": "EU-WEST-1",
                "BYTES_TRANSFERRED": 61235879,
                "TRANSFER_TYPE": "COPY",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSCloudTrailNetworkACLPermissiveEntry(Rule):
    id = "AWS.CloudTrail.NetworkACLPermissiveEntry-prototype"
    display_name = "AWS Network ACL Overly Permissive Entry Created"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Persistence:Account Manipulation"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    default_description = "A Network ACL entry that allows access from anywhere was added.\n"
    default_runbook = (
        "Remove the overly permissive Network ACL entry and add a new entry with more restrictive permissions.\n"
    )
    default_reference = "https://docs.aws.amazon.com/vpc/latest/userguide/vpc-network-acls.html#nacl-rules"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]

    def rule(self, event):
        # Only check successful actions creating a new Network ACL entry
        if not aws_cloudtrail_success(event) or event.get("eventName") != "CreateNetworkAclEntry":
            return False
        # Check if this new NACL entry is allowing traffic from anywhere
        return (
            event.deep_get("requestParameters", "cidrBlock") == "0.0.0.0/0"
            and event.deep_get("requestParameters", "ruleAction") == "allow"
            and (event.deep_get("requestParameters", "egress") is False)
        )

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Overly Permissive Entry Added",
            expected_result=True,
            log={
                "awsRegion": "us-west-2",
                "eventID": "1111",
                "eventName": "CreateNetworkAclEntry",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "aclProtocol": "6",
                    "cidrBlock": "0.0.0.0/0",
                    "egress": False,
                    "icmpTypeCode": {},
                    "networkAclId": "acl-1111",
                    "portRange": {"from": 700, "to": 702},
                    "ruleAction": "allow",
                    "ruleNumber": 12,
                },
                "responseElements": {"_return": True, "requestId": "1111"},
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Not Overly Permissive Entry Added",
            expected_result=False,
            log={
                "awsRegion": "us-west-2",
                "eventID": "1111",
                "eventName": "CreateNetworkAclEntry",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "aclProtocol": "6",
                    "cidrBlock": "111.111.111.111/32",
                    "egress": False,
                    "icmpTypeCode": {},
                    "networkAclId": "acl-1111",
                    "portRange": {"from": 700, "to": 702},
                    "ruleAction": "allow",
                    "ruleNumber": 12,
                },
                "responseElements": {"_return": True, "requestId": "1111"},
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Error Adding Overly Permissive Entry",
            expected_result=False,
            log={
                "awsRegion": "us-west-2",
                "errorCode": "ValidationError",
                "eventID": "1111",
                "eventName": "CreateNetworkAclEntry",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "aclProtocol": "6",
                    "cidrBlock": "0.0.0.0/0",
                    "egress": False,
                    "icmpTypeCode": {},
                    "networkAclId": "acl-1111",
                    "portRange": {"from": 700, "to": 702},
                    "ruleAction": "allow",
                    "ruleNumber": 12,
                },
                "responseElements": {"_return": True, "requestId": "1111"},
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther.rules.gsuite_activityevent.google_workspace_admin_custom_role import (
    GoogleWorkspaceAdminCustomRole as GoogleWorkspaceAdminCustomRole,
)
from pypanther.rules.gsuite_activityevent.google_workspace_advanced_protection_program import (
    GoogleWorkspaceAdvancedProtectionProgram as GoogleWorkspaceAdvancedProtectionProgram,
)
from pypanther.rules.gsuite_activityevent.google_workspace_apps_marketplace_allowlist import (
    GoogleWorkspaceAppsMarketplaceAllowlist as GoogleWorkspaceAppsMarketplaceAllowlist,
)
from pypanther.rules.gsuite_activityevent.google_workspace_apps_marketplace_new_domain_application import (
    GoogleWorkspaceAppsMarketplaceNewDomainApplication as GoogleWorkspaceAppsMarketplaceNewDomainApplication,
)
from pypanther.rules.gsuite_activityevent.google_workspace_apps_new_mobile_app_installed import (
    GoogleWorkspaceAppsNewMobileAppInstalled as GoogleWorkspaceAppsNewMobileAppInstalled,
)
from pypanther.rules.gsuite_activityevent.gsuite_advanced_protection import (
    GSuiteAdvancedProtection as GSuiteAdvancedProtection,
)
from pypanther.rules.gsuite_activityevent.gsuite_calendar_made_public import (
    GSuiteCalendarMadePublic as GSuiteCalendarMadePublic,
)
from pypanther.rules.gsuite_activityevent.gsuite_doc_ownership_transfer import (
    GSuiteDocOwnershipTransfer as GSuiteDocOwnershipTransfer,
)
from pypanther.rules.gsuite_activityevent.gsuite_external_forwarding import (
    GSuiteExternalMailForwarding as GSuiteExternalMailForwarding,
)
from pypanther.rules.gsuite_activityevent.gsuite_google_access import GSuiteGoogleAccess as GSuiteGoogleAccess
from pypanther.rules.gsuite_activityevent.gsuite_gov_attack import (
    GSuiteGovernmentBackedAttack as GSuiteGovernmentBackedAttack,
)
from pypanther.rules.gsuite_activityevent.gsuite_group_banned_user import GSuiteGroupBannedUser as GSuiteGroupBannedUser
from pypanther.rules.gsuite_activityevent.gsuite_leaked_password import GSuiteLeakedPassword as GSuiteLeakedPassword
from pypanther.rules.gsuite_activityevent.gsuite_login_type import GSuiteLoginType as GSuiteLoginType
from pypanther.rules.gsuite_activityevent.gsuite_mobile_device_compromise import (
    GSuiteDeviceCompromise as GSuiteDeviceCompromise,
)
from pypanther.rules.gsuite_activityevent.gsuite_mobile_device_screen_unlock_fail import (
    GSuiteDeviceUnlockFailure as GSuiteDeviceUnlockFailure,
)
from pypanther.rules.gsuite_activityevent.gsuite_mobile_device_suspicious_activity import (
    GSuiteDeviceSuspiciousActivity as GSuiteDeviceSuspiciousActivity,
)
from pypanther.rules.gsuite_activityevent.gsuite_passthrough_rule import GSuiteRule as GSuiteRule
from pypanther.rules.gsuite_activityevent.gsuite_suspicious_logins import (
    GSuiteSuspiciousLogins as GSuiteSuspiciousLogins,
)
from pypanther.rules.gsuite_activityevent.gsuite_two_step_verification import (
    GSuiteTwoStepVerification as GSuiteTwoStepVerification,
)
from pypanther.rules.gsuite_activityevent.gsuite_user_suspended import GSuiteUserSuspended as GSuiteUserSuspended
from pypanther.rules.gsuite_activityevent.gsuite_workspace_calendar_external_sharing import (
    GSuiteWorkspaceCalendarExternalSharingSetting as GSuiteWorkspaceCalendarExternalSharingSetting,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_data_export_created import (
    GSuiteWorkspaceDataExportCreated as GSuiteWorkspaceDataExportCreated,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_gmail_default_routing_rule import (
    GSuiteWorkspaceGmailDefaultRoutingRuleModified as GSuiteWorkspaceGmailDefaultRoutingRuleModified,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_gmail_enhanced_predelivery_scanning import (
    GSuiteWorkspaceGmailPredeliveryScanningDisabled as GSuiteWorkspaceGmailPredeliveryScanningDisabled,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_gmail_security_sandbox_disabled import (
    GSuiteWorkspaceGmailSecuritySandboxDisabled as GSuiteWorkspaceGmailSecuritySandboxDisabled,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_password_enforce_strong_disabled import (
    GSuiteWorkspacePasswordEnforceStrongDisabled as GSuiteWorkspacePasswordEnforceStrongDisabled,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_password_reuse_enabled import (
    GSuiteWorkspacePasswordReuseEnabled as GSuiteWorkspacePasswordReuseEnabled,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_trusted_domains_allowlist import (
    GSuiteWorkspaceTrustedDomainsAllowlist as GSuiteWorkspaceTrustedDomainsAllowlist,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.sublime import sublime_alert_context


@panther_managed
class SublimeMessageSourceDeletedOrDeactivated(Rule):
    default_description = "A Sublime User disabled or deleted some message source(s)."
    display_name = "Sublime Message Source Deleted Or Deactivated"
    default_runbook = "Assess if this was done by the user for a valid business reason. Be vigilant to re-enable the message source(s) if it's in the best security interest for your organization's security posture."
    default_reference = "https://docs.sublime.security/docs/message-types"
    default_severity = Severity.MEDIUM
    alert_title = "Sublime message source(s) were deleted or deactivated"
    log_types = [LogType.SUBLIME_AUDIT]
    id = "Sublime.Message.Source.Deleted.Or.Deactivated-prototype"
    reports = {"MITRE ATT&CK": ["TA0005:T1562.001"]}
    SUSPICIOUS_EVENTS = ["message_source.deactivate", "message_source.delete"]

    def rule(self, event):
        return event.get("type") in self.SUSPICIOUS_EVENTS

    def alert_context(self, event):
        return sublime_alert_context(event)

    tests = [
        RuleTest(
            name="Message Source Deactivated",
            expected_result=True,
            log={
                "created_at": "2024-09-09 19:33:34.237078000",
                "created_by": {
                    "active": True,
                    "created_at": "2024-08-28 22:05:15.715644000",
                    "email_address": "john.doe@sublime.security",
                    "first_name": "John",
                    "google_oauth_user_id": "",
                    "id": "cd3aedfe-a61f-4e0e-ba30-14dcc7883316",
                    "is_enrolled": True,
                    "last_name": "Doe",
                    "microsoft_oauth_user_id": "",
                    "role": "admin",
                    "updated_at": "2024-08-28 22:05:15.715644000",
                },
                "data": {
                    "request": {
                        "authentication_method": "user_session",
                        "body": '{"mailbox_ids":["493c6e21-7787-419b-bada-7c4f50cbb932"]}',
                        "id": "73444211-31af-42d8-99b4-34a139cf7d4a",
                        "ip": "1.2.3.4",
                        "method": "POST",
                        "path": "/v1/message-sources/febb5bf4-2ead-47b1-b467-0ac729bf6871/deactivate",
                        "query": {},
                        "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/128.0.0.0 Safari/537.36",
                    },
                },
                "id": "084732e5-7704-4bbe-ab5a-77f1aa65a737",
                "type": "message_source.deactivate",
            },
        ),
        RuleTest(
            name="Other Events",
            expected_result=False,
            log={
                "created_at": "2024-09-09 19:33:34.237078000",
                "created_by": {
                    "active": True,
                    "created_at": "2024-08-28 22:05:15.715644000",
                    "email_address": "john.doe@sublime.security",
                    "first_name": "John",
                    "google_oauth_user_id": "",
                    "id": "cd3aedfe-a61f-4e0e-ba30-14dcc7883316",
                    "is_enrolled": True,
                    "last_name": "Doe",
                    "microsoft_oauth_user_id": "",
                    "role": "admin",
                    "updated_at": "2024-08-28 22:05:15.715644000",
                },
                "data": {
                    "request": {
                        "authentication_method": "user_session",
                        "body": '{"mailbox_ids":["493c6e21-7787-419b-bada-7c4f50cbb932"]}',
                        "id": "73444211-31af-42d8-99b4-34a139cf7d4a",
                        "ip": "1.2.3.4",
                        "method": "POST",
                        "path": "/v1/message-sources/febb5bf4-2ead-47b1-b467-0ac729bf6871/deactivate",
                        "query": {},
                        "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/128.0.0.0 Safari/537.36",
                    },
                },
                "id": "084732e5-7704-4bbe-ab5a-77f1aa65a737",
                "type": "rule.deactivate",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPK8SPodCreateOrModifyHostPathVolumeMount(Rule):
    id = "GCP.K8S.Pod.Create.Or.Modify.Host.Path.Volume.Mount-prototype"
    display_name = "GCP K8S Pod Create Or Modify Host Path Volume Mount"
    log_types = [LogType.GCP_AUDIT_LOG]
    default_severity = Severity.HIGH
    default_description = "This detection monitors for pod creation with a hostPath volume mount. The attachment to a node's volume can allow for privilege escalation through underlying vulnerabilities or it can open up possibilities for data exfiltration or unauthorized file access. It is very rare to see this being a pod requirement.\n"
    default_runbook = "Investigate the reason of adding hostPath volume mount. Advise that it is discouraged practice. Create ticket if appropriate.\n"
    default_reference = "https://linuxhint.com/kubernetes-hostpath-volumes/"
    reports = {"MITRE ATT&CK": ["TA0010:T1041", "TA0004:T1611"]}
    dedup_period_minutes = 360
    SUSPICIOUS_PATHS = [
        "/var/run/docker.sock",
        "/var/run/crio/crio.sock",
        "/var/lib/kubelet",
        "/var/lib/kubelet/pki",
        "/var/lib/docker/overlay2",
        "/etc/kubernetes",
        "/etc/kubernetes/manifests",
        "/etc/kubernetes/pki",
        "/home/admin",
    ]

    def rule(self, event):
        if event.deep_get("protoPayload", "response", "status") == "Failure":
            return False
        if event.deep_get("protoPayload", "methodName") not in (
            "io.k8s.core.v1.pods.create",
            "io.k8s.core.v1.pods.update",
            "io.k8s.core.v1.pods.patch",
        ):
            return False
        volume_mount_path = event.deep_walk("protoPayload", "request", "spec", "volumes", "hostPath", "path")
        if not volume_mount_path or (
            volume_mount_path not in self.SUSPICIOUS_PATHS
            and (not any(path in self.SUSPICIOUS_PATHS for path in volume_mount_path))
        ):
            return False
        authorization_info = event.deep_walk("protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if (
                auth.get("permission")
                in ("io.k8s.core.v1.pods.create", "io.k8s.core.v1.pods.update", "io.k8s.core.v1.pods.patch")
                and auth.get("granted") is True
            ):
                return True
        return False

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        pod_name = event.deep_get("protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        project_id = event.deep_get("resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] created k8s pod [{pod_name}] with a hostPath volume mount in project [{project_id}]"

    def dedup(self, event):
        return event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")

    def alert_context(self, event):
        context = gcp_alert_context(event)
        volume_mount_path = event.deep_walk("protoPayload", "request", "spec", "volumes", "hostPath", "path")
        context["volume_mount_path"] = volume_mount_path
        return context

    tests = [
        RuleTest(
            name="Pod With Suspicious Volume Mount Created",
            expected_result=True,
            log={
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "some.user@company.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "io.k8s.core.v1.pods.create",
                            "resource": "core/v1/namespaces/default/pods/test",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.pods.create",
                    "request": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {"name": "test", "namespace": "default"},
                        "spec": {
                            "containers": [
                                {
                                    "image": "nginx",
                                    "imagePullPolicy": "Always",
                                    "name": "test",
                                    "volumeMounts": [{"mountPath": "/test", "name": "test-volume"}],
                                },
                            ],
                            "volumes": [
                                {
                                    "hostPath": {"path": "/var/lib/kubelet", "type": "DirectoryOrCreate"},
                                    "name": "test-volume",
                                },
                            ],
                        },
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "kubectl/v1.28.2 (darwin/amd64) kubernetes/89a4ea3",
                    },
                    "resourceName": "core/v1/namespaces/default/pods/test",
                    "response": {
                        "spec": {
                            "containers": [
                                {
                                    "image": "nginx",
                                    "imagePullPolicy": "Always",
                                    "name": "test",
                                    "volumeMounts": [{"mountPath": "/test", "name": "test-volume"}],
                                },
                            ],
                            "volumes": [
                                {
                                    "hostPath": {"path": "/var/lib/kubelet", "type": "DirectoryOrCreate"},
                                    "name": "test-volume",
                                },
                            ],
                        },
                        "status": {"phase": "Pending", "qosClass": "BestEffort"},
                    },
                },
                "receiveTimestamp": "2024-02-16 11:48:43.531373988",
                "resource": {
                    "labels": {
                        "cluster_name": "some-project-cluster",
                        "location": "us-west1",
                        "project_id": "some-project",
                    },
                    "type": "k8s_cluster",
                },
                "timestamp": "2024-02-16 11:48:22.742154000",
            },
        ),
        RuleTest(
            name="Pod With Non-Suspicious Volume Mount Created",
            expected_result=False,
            log={
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "some.user@company.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "io.k8s.core.v1.pods.create",
                            "resource": "core/v1/namespaces/default/pods/test",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.pods.create",
                    "request": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {"name": "test", "namespace": "default"},
                        "spec": {
                            "containers": [
                                {
                                    "image": "nginx",
                                    "imagePullPolicy": "Always",
                                    "name": "test",
                                    "volumeMounts": [{"mountPath": "/test", "name": "test-volume"}],
                                },
                            ],
                            "volumes": [
                                {"hostPath": {"path": "/data", "type": "DirectoryOrCreate"}, "name": "test-volume"},
                            ],
                        },
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "kubectl/v1.28.2 (darwin/amd64) kubernetes/89a4ea3",
                    },
                    "resourceName": "core/v1/namespaces/default/pods/test",
                    "response": {
                        "spec": {
                            "containers": [
                                {
                                    "image": "nginx",
                                    "imagePullPolicy": "Always",
                                    "name": "test",
                                    "volumeMounts": [{"mountPath": "/test", "name": "test-volume"}],
                                },
                            ],
                            "volumes": [
                                {"hostPath": {"path": "/data", "type": "DirectoryOrCreate"}, "name": "test-volume"},
                            ],
                        },
                        "status": {"phase": "Pending", "qosClass": "BestEffort"},
                    },
                },
                "receiveTimestamp": "2024-02-16 11:48:43.531373988",
                "resource": {
                    "labels": {
                        "cluster_name": "some-project-cluster",
                        "location": "us-west1",
                        "project_id": "some-project",
                    },
                    "type": "k8s_cluster",
                },
                "timestamp": "2024-02-16 11:48:22.742154000",
            },
        ),
        RuleTest(
            name="Pod Not Created",
            expected_result=False,
            log={
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "some.user@company.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "io.k8s.core.v1.pods.create",
                            "resource": "core/v1/namespaces/default/pods/test",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.pods.create",
                    "request": {
                        "@type": "core.k8s.io/v1.Pod",
                        "apiVersion": "v1",
                        "kind": "Pod",
                        "metadata": {"name": "test", "namespace": "default"},
                        "spec": {
                            "containers": [
                                {
                                    "image": "nginx",
                                    "imagePullPolicy": "Always",
                                    "name": "test",
                                    "volumeMounts": [{"mountPath": "/test", "name": "test-volume"}],
                                },
                            ],
                            "volumes": [
                                {
                                    "hostPath": {"path": "/var/lib/kubelet", "type": "DirectoryOrCreate"},
                                    "name": "test-volume",
                                },
                            ],
                        },
                        "status": {},
                    },
                    "resourceName": "core/v1/namespaces/default/pods/test",
                    "response": {"status": "Failure"},
                },
                "receiveTimestamp": "2024-02-16 12:55:17.003485190",
                "resource": {
                    "labels": {
                        "cluster_name": "some-project-cluster",
                        "location": "us-west1",
                        "project_id": "some-project",
                    },
                    "type": "k8s_cluster",
                },
                "timestamp": "2024-02-16 12:55:00.510160000",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GitHubBranchPolicyOverride(Rule):
    id = "GitHub.Branch.PolicyOverride-prototype"
    display_name = "GitHub Branch Protection Policy Override"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Initial Access:Supply Chain Compromise"]
    reports = {"MITRE ATT&CK": ["TA0001:T1195"]}
    default_severity = Severity.HIGH
    default_description = "Bypassing branch protection controls could indicate malicious use of admin credentials in an attempt to hide activity."
    default_runbook = "Verify that the GitHub admin performed this activity and validate its use."
    default_reference = "https://docs.github.com/en/repositories/configuring-branches-and-merges-in-your-repository/managing-protected-branches/managing-a-branch-protection-rule"

    def rule(self, event):
        return event.get("action") == "protected_branch.policy_override"

    def title(self, event):
        return f"A branch protection requirement in the repository [{event.get('repo', '<UNKNOWN_REPO>')}] was overridden by user [{event.udm('actor_user')}]"

    tests = [
        RuleTest(
            name="GitHub - Branch Protection Policy Override",
            expected_result=True,
            log={
                "actor": "cat",
                "action": "protected_branch.policy_override",
                "created_at": 1621305118553,
                "p_log_type": "GitHub.Audit",
                "org": "my-org",
                "repo": "my-org/my-repo",
            },
        ),
        RuleTest(
            name="GitHub - Protected Branch Name Updated",
            expected_result=False,
            log={
                "actor": "cat",
                "action": "protected_branch.update_name",
                "created_at": 1621305118553,
                "org": "my-org",
                "p_log_type": "GitHub.Audit",
                "repo": "my-org/my-repo",
            },
        ),
    ]



from pypanther.rules.indicator_creation.new_aws_account_logging import (
    StandardNewAWSAccountCreated as StandardNewAWSAccountCreated,
)
from pypanther.rules.indicator_creation.new_user_account_logging import (
    StandardNewUserAccountCreated as StandardNewUserAccountCreated,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSS3BucketPolicyModified(Rule):
    id = "AWS.S3.BucketPolicyModified-prototype"
    display_name = "AWS S3 Bucket Policy Modified"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Identity & Access Management", "Exfiltration:Exfiltration Over Web Service"]
    reports = {"CIS": ["3.8"], "MITRE ATT&CK": ["TA0010:T1567"]}
    default_severity = Severity.INFO
    dedup_period_minutes = 720
    default_description = "An S3 Bucket was modified.\n"
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-s3-bucket-policy-modified"
    default_reference = "https://docs.aws.amazon.com/AmazonS3/latest/dev/using-iam-policies.html"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "p_any_aws_arns"]
    # API calls that are indicative of KMS CMK Deletion
    S3_POLICY_CHANGE_EVENTS = {
        "PutBucketAcl",
        "PutBucketPolicy",
        "PutBucketCors",
        "PutBucketLifecycle",
        "PutBucketReplication",
        "DeleteBucketPolicy",
        "DeleteBucketCors",
        "DeleteBucketLifecycle",
        "DeleteBucketReplication",
    }

    def rule(self, event):
        return event.get("eventName") in self.S3_POLICY_CHANGE_EVENTS and aws_cloudtrail_success(event)

    def title(self, event):
        return f"S3 bucket modified by [{event.deep_get('userIdentity', 'arn')}]"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="S3 Bucket Policy Modified",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "s3.amazonaws.com",
                "eventName": "PutBucketAcl",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla",
                "requestParameters": {
                    "host": ["bucket.s3.us-west-2.amazonaws.com"],
                    "bucketName": "bucket",
                    "acl": [""],
                    "x-amz-acl": ["private"],
                },
                "responseElements": None,
                "additionalEventData": {
                    "SignatureVersion": "SigV4",
                    "CipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                    "AuthenticationMethod": "AuthHeader",
                },
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="S3 Bucket Policy Modified Error Response",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "s3.amazonaws.com",
                "errorCode": "AccessDenied",
                "eventName": "PutBucketAcl",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla",
                "requestParameters": {
                    "host": ["bucket.s3.us-west-2.amazonaws.com"],
                    "bucketName": "bucket",
                    "acl": [""],
                    "x-amz-acl": ["private"],
                },
                "responseElements": None,
                "additionalEventData": {
                    "SignatureVersion": "SigV4",
                    "CipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                    "AuthenticationMethod": "AuthHeader",
                },
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="S3 Bucket Policy Not Modified",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"},
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "s3.amazonaws.com",
                "eventName": "GetBucketPolicy",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla",
                "requestParameters": {
                    "host": ["bucket.s3.us-west-2.amazonaws.com"],
                    "bucketName": "bucket",
                    "policy": [""],
                },
                "responseElements": None,
                "additionalEventData": {
                    "SignatureVersion": "SigV4",
                    "CipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                    "AuthenticationMethod": "AuthHeader",
                },
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
    ]



from pypanther.rules.carbonblack.cb_audit_admin_grant import CarbonBlackAuditAdminGrant as CarbonBlackAuditAdminGrant
from pypanther.rules.carbonblack.cb_audit_api_key_created_retrieved import (
    CarbonBlackAuditAPIKeyCreatedRetrieved as CarbonBlackAuditAPIKeyCreatedRetrieved,
)
from pypanther.rules.carbonblack.cb_audit_data_forwarder_stopped import (
    CarbonBlackAuditDataForwarderStopped as CarbonBlackAuditDataForwarderStopped,
)
from pypanther.rules.carbonblack.cb_audit_flagged import CarbonBlackAuditFlagged as CarbonBlackAuditFlagged
from pypanther.rules.carbonblack.cb_audit_user_added_outside_org import (
    CarbonBlackAuditUserAddedOutsideOrg as CarbonBlackAuditUserAddedOutsideOrg,
)
from pypanther.rules.carbonblack.cb_passthrough import CarbonBlackAlertV2Passthrough as CarbonBlackAlertV2Passthrough



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.base import deep_get


@panther_managed
class WizAlertPassthrough(Rule):
    id = "Wiz.Alert.Passthrough-prototype"
    default_description = "This rule enriches and contextualizes security alerts generated by Wiz."
    display_name = "Wiz Alert Passthrough Rule"
    default_runbook = "Review the Wiz alert details to determine what malicious behavior was detected, and whether or not it was blocked."
    default_reference = "https://www.wiz.io/product"
    default_severity = Severity.MEDIUM
    log_types = [LogType.WIZ_ISSUES]
    dedup_period_minutes = 720

    def rule(self, event):
        return event.get("status") == "OPEN" and event.get("severity") != "INFORMATIONAL"

    def title(self, event):
        return f"[Wiz Alert]: {event.deep_get('sourceRule', 'name', default='ALERT_DESCRIPTION_NOT_FOUND')}"

    def severity(self, event):
        return event.get("severity")

    def dedup(self, event):
        # For lower-severity events, dedup based on specific source rule to reduce overall alert volume
        if event.get("severity") in ("INFO", "LOW"):
            dedup_str = str(event.deep_get("sourceRule", "id"))
            if dedup_str:
                return dedup_str
        # If the severity is higher, or for some reason we couldn't generate a dedup string based on
        #   the source rule, then use the alert severity + the resource ID itself.
        return event.deep_get("entitySnapshot", "externalId", default="<RESOURCE_NOT_FOUND>") + event.get(
            "severity",
            "<SEVERITY_NOT_FOUND>",
        )

    def description(self, event):
        return event.deep_get("sourceRule", "controlDescription", default="<DESCRIPTION_NOT_FOUND>")

    def runbook(self, event):
        return event.deep_get("sourceRule", "resolutionRecommendation", default="<RECOMMENDATION_NOT_FOUND>")

    def alert_context(self, event):
        security_subcategories = event.deep_get("sourceRule", "securitySubCategories", default=[{}])
        return {
            "id": event.get("id", "<ID_NOT_FOUND>"),
            "type": event.get("type", "<TYPE_NOT_FOUND>"),
            "entity_snapshot": event.get("entitySnapshot", {}),
            "mitre_attack_categories": [
                subcategory
                for subcategory in security_subcategories
                if deep_get(subcategory, "category", "framework", "name") == "MITRE ATT&CK Matrix"
            ],
        }

    tests = [
        RuleTest(
            name="Open Alert",
            expected_result=True,
            log={
                "createdAt": "2024-06-04 02:28:06.763277000",
                "entitySnapshot": {
                    "cloudProviderURL": "",
                    "externalId": "someExternalId",
                    "id": "12345",
                    "name": "someName",
                    "nativeType": "",
                    "providerId": "someProviderId",
                    "region": "",
                    "resourceGroupExternalId": "",
                    "subscriptionExternalId": "",
                    "subscriptionName": "",
                    "tags": {},
                    "type": "DATA_FINDING",
                },
                "id": "54321",
                "notes": [],
                "projects": [
                    {
                        "businessUnit": "",
                        "id": "45678",
                        "name": "Project 2",
                        "riskProfile": {"businessImpact": "MBI"},
                        "slug": "project-2",
                    },
                ],
                "serviceTickets": [],
                "severity": "HIGH",
                "sourceRule": {
                    "__typename": "Control",
                    "controlDescription": "Alert Description",
                    "id": "12345",
                    "name": "Alert Name",
                    "resolutionRecommendation": "Alert Resolution Recommendation",
                    "securitySubCategories": [
                        {
                            "category": {
                                "framework": {"name": "Wiz for Risk Assessment"},
                                "name": "High Profile Threats",
                            },
                            "title": "High-profile vulnerability exploited in the wild",
                        },
                        {
                            "category": {"framework": {"name": "MITRE ATT&CK Matrix"}, "name": "TA0001 Initial Access"},
                            "title": "T1190 Exploit Public-Facing Application",
                        },
                    ],
                },
                "status": "OPEN",
                "statusChangedAt": "2024-06-04 02:28:06.597355000",
                "type": "TOXIC_COMBINATION",
                "updatedAt": "2024-06-04 02:28:06.763277000",
            },
        ),
        RuleTest(
            name="Low-Severity Open Alert",
            expected_result=True,
            log={
                "createdAt": "2024-06-04 02:28:06.763277000",
                "entitySnapshot": {
                    "cloudProviderURL": "",
                    "externalId": "someExternalId",
                    "id": "12345",
                    "name": "someName",
                    "nativeType": "",
                    "providerId": "someProviderId",
                    "region": "",
                    "resourceGroupExternalId": "",
                    "subscriptionExternalId": "",
                    "subscriptionName": "",
                    "tags": {},
                    "type": "DATA_FINDING",
                },
                "id": "54321",
                "notes": [],
                "projects": [
                    {
                        "businessUnit": "",
                        "id": "45678",
                        "name": "Project 2",
                        "riskProfile": {"businessImpact": "MBI"},
                        "slug": "project-2",
                    },
                ],
                "serviceTickets": [],
                "severity": "LOW",
                "sourceRule": {
                    "__typename": "Control",
                    "controlDescription": "Alert Description",
                    "id": "12345",
                    "name": "Alert Name",
                    "resolutionRecommendation": "Alert Resolution Recommendation",
                    "securitySubCategories": [
                        {
                            "category": {
                                "framework": {"name": "Wiz for Risk Assessment"},
                                "name": "High Profile Threats",
                            },
                            "title": "High-profile vulnerability exploited in the wild",
                        },
                        {
                            "category": {"framework": {"name": "MITRE ATT&CK Matrix"}, "name": "TA0001 Initial Access"},
                            "title": "T1190 Exploit Public-Facing Application",
                        },
                    ],
                },
                "status": "OPEN",
                "statusChangedAt": "2024-06-04 02:28:06.597355000",
                "type": "TOXIC_COMBINATION",
                "updatedAt": "2024-06-04 02:28:06.763277000",
            },
        ),
        RuleTest(
            name="Resolved Alert",
            expected_result=False,
            log={
                "createdAt": "2024-06-04 02:28:06.763277000",
                "entitySnapshot": {
                    "cloudProviderURL": "",
                    "externalId": "someExternalId",
                    "id": "12345",
                    "name": "someName",
                    "nativeType": "",
                    "providerId": "someProviderId",
                    "region": "",
                    "resourceGroupExternalId": "",
                    "subscriptionExternalId": "",
                    "subscriptionName": "",
                    "tags": {},
                    "type": "DATA_FINDING",
                },
                "id": "54321",
                "notes": [],
                "projects": [
                    {
                        "businessUnit": "",
                        "id": "45678",
                        "name": "Project 2",
                        "riskProfile": {"businessImpact": "MBI"},
                        "slug": "project-2",
                    },
                ],
                "serviceTickets": [],
                "severity": "HIGH",
                "sourceRule": {
                    "__typename": "Control",
                    "controlDescription": "Alert Description",
                    "id": "12345",
                    "name": "Alert Name",
                    "resolutionRecommendation": "Alert Resolution Recommendation",
                },
                "status": "RESOLVED",
                "statusChangedAt": "2024-06-04 02:28:06.597355000",
                "type": "TOXIC_COMBINATION",
                "updatedAt": "2024-06-04 02:28:06.763277000",
            },
        ),
    ]



from json import loads

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers import event_type
from pypanther.helpers.aws import lookup_aws_account_name
from pypanther.helpers.base import add_parse_delay
from pypanther.helpers.ipinfo import PantherIPInfoException, geoinfo_from_ip


@panther_managed
class StandardBruteForceByUser(Rule):
    id = "Standard.BruteForceByUser-prototype"
    display_name = "Brute Force By User"
    log_types = [
        LogType.ASANA_AUDIT,
        LogType.ATLASSIAN_AUDIT,
        LogType.AWS_CLOUDTRAIL,
        LogType.BOX_EVENT,
        LogType.GSUITE_REPORTS,
        LogType.OKTA_SYSTEM_LOG,
        LogType.ONELOGIN_EVENTS,
        LogType.ONEPASSWORD_SIGN_IN_ATTEMPT,
    ]
    default_severity = Severity.INFO
    reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    default_description = "An actor user was denied login access more times than the configured threshold."
    threshold = 20
    default_reference = "https://owasp.org/www-community/controls/Blocking_Brute_Force_Attacks"
    default_runbook = "Analyze the user ID who failed to login, and other actions taken before/after. Check if this user eventually authenticated successfully."
    summary_attributes = ["p_any_usernames"]
    tags = ["DataModel", "Credential Access:Brute Force"]

    def rule(self, event):
        # filter events on unified data model field
        return event.udm("event_type") == event_type.FAILED_LOGIN

    def title(self, event):
        # use unified data model field in title
        log_type = event.get("p_log_type")
        title_str = f"{log_type}: User [{event.udm('actor_user')}] has exceeded the failed logins threshold"
        if log_type == "AWS.CloudTrail":
            title_str += f" in [{lookup_aws_account_name(event.get('recipientAccountId'))}]"
        return title_str

    def alert_context(self, event):
        try:
            geoinfo = geoinfo_from_ip(event=event, match_field=event.udm_path("source_ip"))
        except PantherIPInfoException:
            geoinfo = {}
        if isinstance(geoinfo, str):
            geoinfo = loads(geoinfo)
        context = {}
        context["geolocation"] = f"{geoinfo.get('city')}, {geoinfo.get('region')} in {geoinfo.get('country')}"
        context["ip"] = geoinfo.get("ip")
        context["reverse_lookup"] = geoinfo.get("hostname", "No reverse lookup hostname")
        context["ip_org"] = geoinfo.get("org", "No organization listed")
        try:
            context = add_parse_delay(event, context)
        except TypeError:
            pass
        except AttributeError:
            pass
        return context

    tests = [
        RuleTest(
            name="AWS.CloudTrail - Successful Login",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "1111",
                    "arn": "arn:aws:iam::123456789012:user/tester",
                    "accountId": "123456789012",
                    "userName": "testuser",
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "signin.amazonaws.com",
                "eventName": "ConsoleLogin",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla",
                "requestParameters": None,
                "responseElements": {"ConsoleLogin": "Success"},
                "additionalEventData": {
                    "LoginTo": "https://console.aws.amazon.com/console/",
                    "MobileVersion": "No",
                    "MFAUsed": "No",
                },
                "eventID": "1",
                "eventType": "AwsConsoleSignIn",
                "recipientAccountId": "123456789012",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="AWS.CloudTrail - Failed Login",
            expected_result=True,
            mocks=[
                RuleMock(
                    object_name="geoinfo_from_ip",
                    return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
                ),
            ],
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "1111",
                    "arn": "arn:aws:iam::123456789012:user/tester",
                    "accountId": "123456789012",
                    "userName": "tester",
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "signin.amazonaws.com",
                "eventName": "ConsoleLogin",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla",
                "requestParameters": None,
                "responseElements": {"ConsoleLogin": "Failure"},
                "additionalEventData": {
                    "LoginTo": "https://console.aws.amazon.com/console/",
                    "MobileVersion": "No",
                    "MFAUsed": "No",
                },
                "eventID": "1",
                "eventType": "AwsConsoleSignIn",
                "recipientAccountId": "123456789012",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="Box - Regular Event",
            expected_result=False,
            log={
                "type": "event",
                "additional_details": '{"key": "value"}',
                "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
                "ip_address": "111.111.111.111",
                "event_type": "DELETE",
                "p_log_type": "Box.Event",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="Box - Login Failed",
            expected_result=True,
            mocks=[
                RuleMock(
                    object_name="geoinfo_from_ip",
                    return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
                ),
            ],
            log={
                "type": "event",
                "additional_details": '{"key": "value"}',
                "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
                "event_type": "FAILED_LOGIN",
                "source": {"id": "12345678", "type": "user", "name": "Bob Cat"},
                "ip_address": "111.111.111.111",
                "p_log_type": "Box.Event",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="GSuite - Normal Login Event",
            expected_result=False,
            log={
                "id": {"applicationName": "login"},
                "ipAddress": "111.111.111.111",
                "events": [{"type": "login", "name": "login_success"}],
                "p_log_type": "GSuite.Reports",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="GSuite - Failed Login Event",
            expected_result=True,
            mocks=[
                RuleMock(
                    object_name="geoinfo_from_ip",
                    return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
                ),
            ],
            log={
                "actor": {"email": "bob@example.com"},
                "id": {"applicationName": "login"},
                "ipAddress": "111.111.111.111",
                "events": [{"type": "login", "name": "login_failure"}],
                "p_log_type": "GSuite.Reports",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="Okta - Successful Login",
            expected_result=False,
            log={
                "actor": {"alternateId": "admin", "displayName": "unknown", "id": "unknown", "type": "User"},
                "client": {"ipAddress": "111.111.111.111"},
                "eventType": "user.session.start",
                "outcome": {"reason": "VERIFICATION_ERROR", "result": "SUCCESS"},
                "p_log_type": "Okta.SystemLog",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="Okta - Failed Login",
            expected_result=True,
            mocks=[
                RuleMock(
                    object_name="geoinfo_from_ip",
                    return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
                ),
            ],
            log={
                "actor": {"alternateId": "admin", "displayName": "unknown", "id": "unknown", "type": "User"},
                "client": {"ipAddress": "111.111.111.111"},
                "eventType": "user.session.start",
                "outcome": {"reason": "VERIFICATION_ERROR", "result": "FAILURE"},
                "p_log_type": "Okta.SystemLog",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="OneLogin - Normal Login Event",
            expected_result=False,
            log={
                "event_type_id": 8,
                "actor_user_id": 123456,
                "actor_user_name": "Bob Cat",
                "user_id": 123456,
                "user_name": "Bob Cat",
                "ipaddr": "111.111.111.111",
                "p_log_type": "OneLogin.Events",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="OneLogin - Failed Login Event",
            expected_result=True,
            mocks=[
                RuleMock(
                    object_name="geoinfo_from_ip",
                    return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
                ),
            ],
            log={
                "event_type_id": 6,
                "actor_user_id": 123456,
                "actor_user_name": "Bob Cat",
                "user_id": 123456,
                "user_name": "Bob Cat",
                "ipaddr": "1.2.3.4",
                "p_log_type": "OneLogin.Events",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="GCP - Non Login Event",
            expected_result=False,
            log={
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "serviceName": "cloudresourcemanager.googleapis.com",
                    "methodName": "SetIamPolicy",
                    "authenticationInfo": {"principalEmail": "bob@example.com"},
                    "requestMetadata": {"callerIP": "111.111.111.111"},
                    "serviceData": {
                        "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                        "policyDelta": {
                            "bindingDeltas": [
                                {
                                    "action": "ADD",
                                    "member": "cat@example.com",
                                    "role": "roles/resourcemanager.organizationAdmin",
                                },
                            ],
                        },
                    },
                },
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="Asana - Failed Login",
            expected_result=True,
            mocks=[
                RuleMock(
                    object_name="geoinfo_from_ip",
                    return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
                ),
            ],
            log={
                "actor": {"actor_type": "user", "email": "homer@springfield.com", "gid": "2222222", "name": "Homer"},
                "context": {"client_ip_address": "8.8.8.8", "context_type": "web"},
                "created_at": "2021-10-21T23:38:10.364Z",
                "details": {"method": ["ONE_TIME_KEY"]},
                "event_category": "logins",
                "event_type": "user_login_failed",
                "gid": "222222222",
                "resource": {
                    "email": "homer@springfield.com",
                    "gid": "2222222",
                    "name": "homer",
                    "resource_type": "user",
                },
                "p_log_type": "Asana.Audit",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="Asana - Normal Login",
            expected_result=False,
            log={
                "actor": {"actor_type": "user", "email": "homer@springfield.com", "gid": "2222222", "name": "Homer"},
                "context": {"client_ip_address": "8.8.8.8", "context_type": "web"},
                "created_at": "2021-10-21T23:38:10.364Z",
                "details": {"method": ["ONE_TIME_KEY"]},
                "event_category": "logins",
                "event_type": "user_login_succeeded",
                "gid": "222222222",
                "resource": {
                    "email": "homer@springfield.com",
                    "gid": "2222222",
                    "name": "homer",
                    "resource_type": "user",
                },
                "p_log_type": "Asana.Audit",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
        RuleTest(
            name="1Password - Regular Login",
            expected_result=False,
            log={
                "uuid": "1234",
                "session_uuid": "5678",
                "timestamp": "2021-12-03 19:52:52",
                "category": "success",
                "type": "credentials_ok",
                "country": "US",
                "target_user": {"email": "homer@springfield.gov", "name": "Homer Simpson", "uuid": "1234"},
                "client": {
                    "app_name": "1Password Browser Extension",
                    "app_version": "20184",
                    "ip_address": "1.1.1.1",
                    "os_name": "Solaris",
                    "os_version": "10",
                    "platform_name": "Chrome",
                    "platform_version": "96.0.4664.55",
                },
                "p_log_type": "OnePassword.SignInAttempt",
            },
        ),
        RuleTest(
            name="1Password - Failed Login",
            expected_result=True,
            mocks=[
                RuleMock(
                    object_name="geoinfo_from_ip",
                    return_value='{ "ip": "111.111.111.111", "region": "UnitTestRegion", "city": "UnitTestCityNew", "country": "UnitTestCountry", "hostname": "somedomain.com", "org": "Some Org" }',
                ),
            ],
            log={
                "uuid": "1234",
                "session_uuid": "5678",
                "timestamp": "2021-12-03 19:52:52",
                "category": "credentials_failed",
                "type": "password_secret_bad",
                "country": "US",
                "target_user": {"email": "homer@springfield.gov", "name": "Homer Simpson", "uuid": "1234"},
                "client": {
                    "app_name": "1Password Browser Extension",
                    "app_version": "20184",
                    "ip_address": "111.111.111.111",
                    "os_name": "Solaris",
                    "os_version": "10",
                    "platform_name": "Chrome",
                    "platform_version": "96.0.4664.55",
                },
                "p_log_type": "OnePassword.SignInAttempt",
                "p_parse_time": "2021-06-04 10:02:33.650807",
                "p_event_time": "2021-06-04 09:59:53.650807",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class NetskopeManyDeletes(Rule):
    id = "Netskope.ManyDeletes-prototype"
    display_name = "Netskope Many Objects Deleted"
    log_types = [LogType.NETSKOPE_AUDIT]
    tags = ["Netskope", "Configuration Required", "Data Destruction"]
    reports = {"MITRE ATT&CK": ["TA0040:T1485"]}
    default_severity = Severity.HIGH
    default_description = "A user deleted a large number of objects in a short period of time."
    threshold = 10
    default_runbook = "A user deleted a large number of objects in a short period of time.  Validate that this activity is expected and authorized."
    default_reference = "https://docs.netskope.com/en/netskope-help/admin-console/administration/audit-log/"

    def rule(self, event):
        audit_log_event = event.get("audit_log_event")
        if audit_log_event and "Delete" in audit_log_event:
            return True
        return False

    def title(self, event):
        user = event.get("user", "<USER_NOT_FOUND>")
        return f"[{user}] deleted many objects in a short time"

    tests = [
        RuleTest(
            name="True positive",
            expected_result=True,
            log={
                "_id": "1e589befa3da30132362f32a",
                "_insertion_epoch_timestamp": 1702318213,
                "audit_log_event": "Deleted rbi template",
                "count": 1,
                "is_netskope_personnel": False,
                "organization_unit": "",
                "severity_level": 2,
                "timestamp": "2023-12-11 18:10:13.000000000",
                "type": "admin_audit_logs",
                "ur_normalized": "service-account",
                "user": "service-account",
            },
        ),
        RuleTest(
            name="True negative",
            expected_result=False,
            log={
                "_id": "1e589befa3da30132362f32a",
                "_insertion_epoch_timestamp": 1702318213,
                "audit_log_event": "Rest API V2 Call",
                "count": 1,
                "is_netskope_personnel": False,
                "organization_unit": "",
                "severity_level": 2,
                "supporting_data": {
                    "data_type": "incidents",
                    "data_values": [200, "POST", "/api/v2/incidents/uba/getuci", "trid=ccb898fgrhvdd0v0lebg"],
                },
                "timestamp": "2023-12-11 18:10:13.000000000",
                "type": "admin_audit_logs",
                "ur_normalized": "service-account",
                "user": "service-account",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSECREVENTS(Rule):
    id = "AWS.ECR.EVENTS-prototype"
    display_name = "AWS ECR Events"
    enabled = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Security Control", "Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0005:T1535"]}
    default_severity = Severity.INFO
    create_alert = False
    default_description = "An ECR event occurred outside of an expected account or region"
    default_runbook = "https://docs.aws.amazon.com/AmazonECR/latest/userguide/logging-using-cloudtrail.html"
    default_reference = (
        "https://aws.amazon.com/blogs/containers/amazon-ecr-in-multi-account-and-multi-region-architectures/"
    )
    summary_attributes = ["eventSource", "recipientAccountId", "awsRegion", "p_any_aws_arns"]
    # CONFIGURATION REQUIRED: Update with your expected AWS Accounts/Regions
    AWS_ACCOUNTS_AND_REGIONS = {"123456789012": {"us-west-1", "us-west-2"}, "103456789012": {"us-east-1", "us-east-2"}}

    def rule(self, event):
        if event.get("eventSource") == "ecr.amazonaws.com":
            aws_account_id = event.deep_get("userIdentity", "accountId")
            if aws_account_id in self.AWS_ACCOUNTS_AND_REGIONS:
                if event.get("awsRegion") not in self.AWS_ACCOUNTS_AND_REGIONS[aws_account_id]:
                    return True
            else:
                return True
        return False

    def dedup(self, event):
        return event.get("recipientAccountId")

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Authorized account, unauthorized region",
            expected_result=True,
            log={
                "eventVersion": "1.04",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "AIDACKCEVSQ6C2EXAMPLE:account_name",
                    "arn": "arn:aws:sts::123456789012:user/Mary_Major",
                    "accountId": "123456789012",
                    "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                    "userName": "Mary_Major",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-04-15T16:42:14Z"},
                    },
                },
                "eventTime": "2019-04-15T16:45:00Z",
                "eventSource": "ecr.amazonaws.com",
                "eventName": "PutImage",
                "awsRegion": "us-east-2",
                "sourceIPAddress": "203.0.113.12",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {
                    "repositoryName": "testrepo",
                    "imageTag": "latest",
                    "registryId": "123456789012",
                    "imageManifest": '{\n   "schemaVersion": 2,\n   "mediaType": "application/vnd.docker.distribution.manifest.v2+json",\n   "config": {\n      "mediaType": "application/vnd.docker.container.image.v1+json",\n      "size": 5543,\n      "digest": "sha256:000b9b805af1cdb60628898c9f411996301a1c13afd3dbef1d8a16ac6dbf503a"\n   },\n   "layers": [\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 43252507,\n         "digest": "sha256:3b37166ec61459e76e33282dda08f2a9cd698ca7e3d6bc44e6a6e7580cdeff8e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 846,\n         "digest": "sha256:504facff238fde83f1ca8f9f54520b4219c5b8f80be9616ddc52d31448a044bd"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 615,\n         "digest": "sha256:ebbcacd28e101968415b0c812b2d2dc60f969e36b0b08c073bf796e12b1bb449"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 850,\n         "digest": "sha256:c7fb3351ecad291a88b92b600037e2435c84a347683d540042086fe72c902b8a"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 168,\n         "digest": "sha256:2e3debadcbf7e542e2aefbce1b64a358b1931fb403b3e4aeca27cb4d809d56c2"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 37720774,\n         "digest": "sha256:f8c9f51ad524d8ae9bf4db69cd3e720ba92373ec265f5c390ffb21bb0c277941"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 30432107,\n         "digest": "sha256:813a50b13f61cf1f8d25f19fa96ad3aa5b552896c83e86ce413b48b091d7f01b"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 197,\n         "digest": "sha256:7ab043301a6187ea3293d80b30ba06c7bf1a0c3cd4c43d10353b31bc0cecfe7d"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 154,\n         "digest": "sha256:67012cca8f31dc3b8ee2305e7762fee20c250513effdedb38a1c37784a5a2e71"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 176,\n         "digest": "sha256:3bc892145603fffc9b1c97c94e2985b4cb19ca508750b15845a5d97becbd1a0e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 183,\n         "digest": "sha256:6f1c79518f18251d35977e7e46bfa6c6b9cf50df2a79d4194941d95c54258d18"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:b7bcfbc2e2888afebede4dd1cd5eebf029bb6315feeaf0b56e425e11a50afe42"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:2b220f8b0f32b7c2ed8eaafe1c802633bbd94849b9ab73926f0ba46cdae91629"\n      }\n   ]\n}',
                },
                "responseElements": {
                    "image": {
                        "repositoryName": "testrepo",
                        "imageManifest": '{\n   "schemaVersion": 2,\n   "mediaType": "application/vnd.docker.distribution.manifest.v2+json",\n   "config": {\n      "mediaType": "application/vnd.docker.container.image.v1+json",\n      "size": 5543,\n      "digest": "sha256:000b9b805af1cdb60628898c9f411996301a1c13afd3dbef1d8a16ac6dbf503a"\n   },\n   "layers": [\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 43252507,\n         "digest": "sha256:3b37166ec61459e76e33282dda08f2a9cd698ca7e3d6bc44e6a6e7580cdeff8e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 846,\n         "digest": "sha256:504facff238fde83f1ca8f9f54520b4219c5b8f80be9616ddc52d31448a044bd"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 615,\n         "digest": "sha256:ebbcacd28e101968415b0c812b2d2dc60f969e36b0b08c073bf796e12b1bb449"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 850,\n         "digest": "sha256:c7fb3351ecad291a88b92b600037e2435c84a347683d540042086fe72c902b8a"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 168,\n         "digest": "sha256:2e3debadcbf7e542e2aefbce1b64a358b1931fb403b3e4aeca27cb4d809d56c2"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 37720774,\n         "digest": "sha256:f8c9f51ad524d8ae9bf4db69cd3e720ba92373ec265f5c390ffb21bb0c277941"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 30432107,\n         "digest": "sha256:813a50b13f61cf1f8d25f19fa96ad3aa5b552896c83e86ce413b48b091d7f01b"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 197,\n         "digest": "sha256:7ab043301a6187ea3293d80b30ba06c7bf1a0c3cd4c43d10353b31bc0cecfe7d"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 154,\n         "digest": "sha256:67012cca8f31dc3b8ee2305e7762fee20c250513effdedb38a1c37784a5a2e71"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 176,\n         "digest": "sha256:3bc892145603fffc9b1c97c94e2985b4cb19ca508750b15845a5d97becbd1a0e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 183,\n         "digest": "sha256:6f1c79518f18251d35977e7e46bfa6c6b9cf50df2a79d4194941d95c54258d18"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:b7bcfbc2e2888afebede4dd1cd5eebf029bb6315feeaf0b56e425e11a50afe42"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:2b220f8b0f32b7c2ed8eaafe1c802633bbd94849b9ab73926f0ba46cdae91629"\n      }\n   ]\n}',
                        "registryId": "123456789012",
                        "imageId": {
                            "imageDigest": "sha256:98c8b060c21d9adbb6b8c41b916e95e6307102786973ab93a41e8b86d1fc6d3e",
                            "imageTag": "latest",
                        },
                    },
                },
                "requestID": "cf044b7d-5f9d-11e9-9b2a-95983139cc57",
                "eventID": "2bfd4ee2-2178-4a82-a27d-b12939923f0f",
                "resources": [
                    {"ARN": "arn:aws:ecr:us-east-2:123456789012:repository/testrepo", "accountId": "123456789012"},
                ],
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Unauthorized account",
            expected_result=True,
            log={
                "eventVersion": "1.04",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "AIDACKCEVSQ6C2EXAMPLE:account_name",
                    "arn": "arn:aws:sts::123456789000:user/Mary_Major",
                    "accountId": "123456789000",
                    "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                    "userName": "Mary_Major",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-04-15T16:42:14Z"},
                    },
                },
                "eventTime": "2019-04-15T16:45:00Z",
                "eventSource": "ecr.amazonaws.com",
                "eventName": "PutImage",
                "awsRegion": "us-east-2",
                "sourceIPAddress": "203.0.113.12",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {
                    "repositoryName": "testrepo",
                    "imageTag": "latest",
                    "registryId": "123456789000",
                    "imageManifest": '{\n   "schemaVersion": 2,\n   "mediaType": "application/vnd.docker.distribution.manifest.v2+json",\n   "config": {\n      "mediaType": "application/vnd.docker.container.image.v1+json",\n      "size": 5543,\n      "digest": "sha256:000b9b805af1cdb60628898c9f411996301a1c13afd3dbef1d8a16ac6dbf503a"\n   },\n   "layers": [\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 43252507,\n         "digest": "sha256:3b37166ec61459e76e33282dda08f2a9cd698ca7e3d6bc44e6a6e7580cdeff8e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 846,\n         "digest": "sha256:504facff238fde83f1ca8f9f54520b4219c5b8f80be9616ddc52d31448a044bd"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 615,\n         "digest": "sha256:ebbcacd28e101968415b0c812b2d2dc60f969e36b0b08c073bf796e12b1bb449"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 850,\n         "digest": "sha256:c7fb3351ecad291a88b92b600037e2435c84a347683d540042086fe72c902b8a"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 168,\n         "digest": "sha256:2e3debadcbf7e542e2aefbce1b64a358b1931fb403b3e4aeca27cb4d809d56c2"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 37720774,\n         "digest": "sha256:f8c9f51ad524d8ae9bf4db69cd3e720ba92373ec265f5c390ffb21bb0c277941"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 30432107,\n         "digest": "sha256:813a50b13f61cf1f8d25f19fa96ad3aa5b552896c83e86ce413b48b091d7f01b"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 197,\n         "digest": "sha256:7ab043301a6187ea3293d80b30ba06c7bf1a0c3cd4c43d10353b31bc0cecfe7d"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 154,\n         "digest": "sha256:67012cca8f31dc3b8ee2305e7762fee20c250513effdedb38a1c37784a5a2e71"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 176,\n         "digest": "sha256:3bc892145603fffc9b1c97c94e2985b4cb19ca508750b15845a5d97becbd1a0e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 183,\n         "digest": "sha256:6f1c79518f18251d35977e7e46bfa6c6b9cf50df2a79d4194941d95c54258d18"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:b7bcfbc2e2888afebede4dd1cd5eebf029bb6315feeaf0b56e425e11a50afe42"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:2b220f8b0f32b7c2ed8eaafe1c802633bbd94849b9ab73926f0ba46cdae91629"\n      }\n   ]\n}',
                },
                "responseElements": {
                    "image": {
                        "repositoryName": "testrepo",
                        "imageManifest": '{\n   "schemaVersion": 2,\n   "mediaType": "application/vnd.docker.distribution.manifest.v2+json",\n   "config": {\n      "mediaType": "application/vnd.docker.container.image.v1+json",\n      "size": 5543,\n      "digest": "sha256:000b9b805af1cdb60628898c9f411996301a1c13afd3dbef1d8a16ac6dbf503a"\n   },\n   "layers": [\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 43252507,\n         "digest": "sha256:3b37166ec61459e76e33282dda08f2a9cd698ca7e3d6bc44e6a6e7580cdeff8e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 846,\n         "digest": "sha256:504facff238fde83f1ca8f9f54520b4219c5b8f80be9616ddc52d31448a044bd"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 615,\n         "digest": "sha256:ebbcacd28e101968415b0c812b2d2dc60f969e36b0b08c073bf796e12b1bb449"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 850,\n         "digest": "sha256:c7fb3351ecad291a88b92b600037e2435c84a347683d540042086fe72c902b8a"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 168,\n         "digest": "sha256:2e3debadcbf7e542e2aefbce1b64a358b1931fb403b3e4aeca27cb4d809d56c2"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 37720774,\n         "digest": "sha256:f8c9f51ad524d8ae9bf4db69cd3e720ba92373ec265f5c390ffb21bb0c277941"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 30432107,\n         "digest": "sha256:813a50b13f61cf1f8d25f19fa96ad3aa5b552896c83e86ce413b48b091d7f01b"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 197,\n         "digest": "sha256:7ab043301a6187ea3293d80b30ba06c7bf1a0c3cd4c43d10353b31bc0cecfe7d"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 154,\n         "digest": "sha256:67012cca8f31dc3b8ee2305e7762fee20c250513effdedb38a1c37784a5a2e71"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 176,\n         "digest": "sha256:3bc892145603fffc9b1c97c94e2985b4cb19ca508750b15845a5d97becbd1a0e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 183,\n         "digest": "sha256:6f1c79518f18251d35977e7e46bfa6c6b9cf50df2a79d4194941d95c54258d18"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:b7bcfbc2e2888afebede4dd1cd5eebf029bb6315feeaf0b56e425e11a50afe42"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:2b220f8b0f32b7c2ed8eaafe1c802633bbd94849b9ab73926f0ba46cdae91629"\n      }\n   ]\n}',
                        "registryId": "123456789000",
                        "imageId": {
                            "imageDigest": "sha256:98c8b060c21d9adbb6b8c41b916e95e6307102786973ab93a41e8b86d1fc6d3e",
                            "imageTag": "latest",
                        },
                    },
                },
                "requestID": "cf044b7d-5f9d-11e9-9b2a-95983139cc57",
                "eventID": "2bfd4ee2-2178-4a82-a27d-b12939923f0f",
                "resources": [
                    {"ARN": "arn:aws:ecr:us-east-2:123456789000:repository/testrepo", "accountId": "123456789000"},
                ],
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789000",
            },
        ),
        RuleTest(
            name="Authorized account",
            expected_result=False,
            log={
                "eventVersion": "1.04",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "AIDACKCEVSQ6C2EXAMPLE:account_name",
                    "arn": "arn:aws:sts::123456789012:user/Mary_Major",
                    "accountId": "123456789012",
                    "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                    "userName": "Mary_Major",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-04-15T16:42:14Z"},
                    },
                },
                "eventTime": "2019-04-15T16:45:00Z",
                "eventSource": "ecr.amazonaws.com",
                "eventName": "PutImage",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "203.0.113.12",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {
                    "repositoryName": "testrepo",
                    "imageTag": "latest",
                    "registryId": "123456789012",
                    "imageManifest": '{\n   "schemaVersion": 2,\n   "mediaType": "application/vnd.docker.distribution.manifest.v2+json",\n   "config": {\n      "mediaType": "application/vnd.docker.container.image.v1+json",\n      "size": 5543,\n      "digest": "sha256:000b9b805af1cdb60628898c9f411996301a1c13afd3dbef1d8a16ac6dbf503a"\n   },\n   "layers": [\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 43252507,\n         "digest": "sha256:3b37166ec61459e76e33282dda08f2a9cd698ca7e3d6bc44e6a6e7580cdeff8e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 846,\n         "digest": "sha256:504facff238fde83f1ca8f9f54520b4219c5b8f80be9616ddc52d31448a044bd"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 615,\n         "digest": "sha256:ebbcacd28e101968415b0c812b2d2dc60f969e36b0b08c073bf796e12b1bb449"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 850,\n         "digest": "sha256:c7fb3351ecad291a88b92b600037e2435c84a347683d540042086fe72c902b8a"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 168,\n         "digest": "sha256:2e3debadcbf7e542e2aefbce1b64a358b1931fb403b3e4aeca27cb4d809d56c2"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 37720774,\n         "digest": "sha256:f8c9f51ad524d8ae9bf4db69cd3e720ba92373ec265f5c390ffb21bb0c277941"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 30432107,\n         "digest": "sha256:813a50b13f61cf1f8d25f19fa96ad3aa5b552896c83e86ce413b48b091d7f01b"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 197,\n         "digest": "sha256:7ab043301a6187ea3293d80b30ba06c7bf1a0c3cd4c43d10353b31bc0cecfe7d"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 154,\n         "digest": "sha256:67012cca8f31dc3b8ee2305e7762fee20c250513effdedb38a1c37784a5a2e71"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 176,\n         "digest": "sha256:3bc892145603fffc9b1c97c94e2985b4cb19ca508750b15845a5d97becbd1a0e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 183,\n         "digest": "sha256:6f1c79518f18251d35977e7e46bfa6c6b9cf50df2a79d4194941d95c54258d18"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:b7bcfbc2e2888afebede4dd1cd5eebf029bb6315feeaf0b56e425e11a50afe42"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:2b220f8b0f32b7c2ed8eaafe1c802633bbd94849b9ab73926f0ba46cdae91629"\n      }\n   ]\n}',
                },
                "responseElements": {
                    "image": {
                        "repositoryName": "testrepo",
                        "imageManifest": '{\n   "schemaVersion": 2,\n   "mediaType": "application/vnd.docker.distribution.manifest.v2+json",\n   "config": {\n      "mediaType": "application/vnd.docker.container.image.v1+json",\n      "size": 5543,\n      "digest": "sha256:000b9b805af1cdb60628898c9f411996301a1c13afd3dbef1d8a16ac6dbf503a"\n   },\n   "layers": [\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 43252507,\n         "digest": "sha256:3b37166ec61459e76e33282dda08f2a9cd698ca7e3d6bc44e6a6e7580cdeff8e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 846,\n         "digest": "sha256:504facff238fde83f1ca8f9f54520b4219c5b8f80be9616ddc52d31448a044bd"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 615,\n         "digest": "sha256:ebbcacd28e101968415b0c812b2d2dc60f969e36b0b08c073bf796e12b1bb449"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 850,\n         "digest": "sha256:c7fb3351ecad291a88b92b600037e2435c84a347683d540042086fe72c902b8a"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 168,\n         "digest": "sha256:2e3debadcbf7e542e2aefbce1b64a358b1931fb403b3e4aeca27cb4d809d56c2"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 37720774,\n         "digest": "sha256:f8c9f51ad524d8ae9bf4db69cd3e720ba92373ec265f5c390ffb21bb0c277941"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 30432107,\n         "digest": "sha256:813a50b13f61cf1f8d25f19fa96ad3aa5b552896c83e86ce413b48b091d7f01b"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 197,\n         "digest": "sha256:7ab043301a6187ea3293d80b30ba06c7bf1a0c3cd4c43d10353b31bc0cecfe7d"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 154,\n         "digest": "sha256:67012cca8f31dc3b8ee2305e7762fee20c250513effdedb38a1c37784a5a2e71"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 176,\n         "digest": "sha256:3bc892145603fffc9b1c97c94e2985b4cb19ca508750b15845a5d97becbd1a0e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 183,\n         "digest": "sha256:6f1c79518f18251d35977e7e46bfa6c6b9cf50df2a79d4194941d95c54258d18"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:b7bcfbc2e2888afebede4dd1cd5eebf029bb6315feeaf0b56e425e11a50afe42"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:2b220f8b0f32b7c2ed8eaafe1c802633bbd94849b9ab73926f0ba46cdae91629"\n      }\n   ]\n}',
                        "registryId": "123456789012",
                        "imageId": {
                            "imageDigest": "sha256:98c8b060c21d9adbb6b8c41b916e95e6307102786973ab93a41e8b86d1fc6d3e",
                            "imageTag": "latest",
                        },
                    },
                },
                "requestID": "cf044b7d-5f9d-11e9-9b2a-95983139cc57",
                "eventID": "2bfd4ee2-2178-4a82-a27d-b12939923f0f",
                "resources": [
                    {"ARN": "arn:aws:ecr:us-east-2:123456789012:repository/testrepo", "accountId": "123456789012"},
                ],
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
    ]



"""
Used to define global helpers and variables for ZenDesk events.
"""
# # # # # # # # # # # # # #
#      Zendesk Helpers     #
# # # # # # # # # # # # # #

import re

# key names
ZENDESK_CHANGE_DESCRIPTION = "change_description"
ZENDESK_APP_ROLE_ASSIGNED = re.compile(
    r"(?P<app>.*) role changed from (?P<old_role>.+) to (?P<new_role>.*)",
    re.IGNORECASE,
)
ZENDESK_ROLE_ASSIGNED = re.compile(
    r"Role changed from (?P<old_role>.+) to (?P<new_role>[^$]+)",
    re.IGNORECASE,
)


def zendesk_get_roles(event):
    old_role = ""
    new_role = ""
    role_change = event.get(ZENDESK_CHANGE_DESCRIPTION, "")
    if "\n" in role_change:
        for app_change in role_change.split("\n"):
            matches = ZENDESK_APP_ROLE_ASSIGNED.match(app_change)
            if matches:
                if old_role:
                    old_role += " ; "
                old_role += matches.group("app") + ":" + matches.group("old_role")
                if new_role:
                    new_role += " ; "
                new_role += matches.group("app") + ":" + matches.group("new_role")
    else:
        matches = ZENDESK_ROLE_ASSIGNED.match(role_change)
        if matches:
            old_role = matches.group("old_role")
            new_role = matches.group("new_role")
    if not old_role:
        old_role = "<UNKNOWN_APP>:<UNKNOWN_ROLE>"
    if not new_role:
        new_role = "<UNKNOWN_APP>:<UNKNOWN_ROLE>"
    return old_role, new_role



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.base import deep_get


@panther_managed
class GCPUnusedRegions(Rule):
    id = "GCP.UnusedRegions-prototype"
    display_name = "GCP Resource in Unused Region"
    enabled = False
    dedup_period_minutes = 15
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Database", "Configuration Required", "Defense Evasion:Unused/Unsupported Cloud Regions"]
    reports = {"MITRE ATT&CK": ["TA0005:T1535"]}
    default_severity = Severity.MEDIUM
    default_description = (
        "Adversaries may create cloud instances in unused geographic service regions in order to evade detection.\n"
    )
    default_runbook = "Validate the user making the request and the resource created."
    default_reference = "https://cloud.google.com/docs/geography-and-regions"
    summary_attributes = ["severity", "p_any_ip_addresses", "p_any_domain_names"]
    # 'asia',
    # 'australia',
    # 'eu',
    # 'northamerica',
    # 'southamerica',
    APPROVED_ACTIVE_REGIONS = {"us"}

    def _resource_in_active_region(self, location):
        # return False if location is None, meaning the event did not have a location attribute
        # in any of the places we would expect to find one.
        if location is False:
            return False
        return not any(location.startswith(active_region) for active_region in self.APPROVED_ACTIVE_REGIONS)

    def _get_location_or_zone(self, event):
        resource = event.get("resource")
        if not resource:
            return False
        resource_location = deep_get(resource, "labels", "location")
        if resource_location:
            return resource_location
        resource_zone = deep_get(resource, "labels", "zone")
        if resource_zone:
            return resource_zone
        return False

    def rule(self, event):
        method_name = event.deep_get("protoPayload", "methodName", default="<UNKNOWN_METHOD>")
        if not method_name.endswith(("insert", "create")):
            return False
        return self._resource_in_active_region(self._get_location_or_zone(event))

    def title(self, event):
        return f"GCP resource(s) created in unused region/zone in project {event.deep_get('resource', 'labels', 'project_id', default='<UNKNOWN_PROJECT>')}"

    tests = [
        RuleTest(
            name="GCE Instance Terminated",
            expected_result=False,
            log={
                "logName": "projects/western-verve-225918/logs/cloudaudit.googleapis.com%2Factivity",
                "severity": "NOTICE",
                "insertId": "81xwjyd5vh0",
                "resource": {
                    "type": "gce_instance",
                    "labels": {
                        "instance_id": "5423650206360236277",
                        "project_id": "western-verve-225918",
                        "zone": "europe-north1-a",
                    },
                },
                "timestamp": "2020-05-15 17:55:19.303000000",
                "receiveTimestamp": "2020-05-15 17:55:20.210068584",
                "operation": {
                    "id": "operation-1589565287793-5a5b382538481-3da9816c-ce929997",
                    "producer": "compute.googleapis.com",
                    "last": True,
                },
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "serviceName": "compute.googleapis.com",
                    "methodName": "v1.compute.instances.delete",
                    "resourceName": "projects/western-verve-225918/zones/europe-north1-a/instances/instance-3",
                    "authenticationInfo": {"principalEmail": "jack.naglieri@runpanther.io"},
                    "requestMetadata": {
                        "callerIP": "136.24.229.58",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe),gzip(gfe)",
                    },
                    "request": {"@type": "type.googleapis.com/compute.instances.delete"},
                },
            },
        ),
        RuleTest(
            name="GCE Create Instance in SouthAmerica",
            expected_result=True,
            log={
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user.name@runpanther.io"},
                    "requestMetadata": {
                        "callerIp": "136.24.229.58",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe)",
                    },
                    "serviceName": "compute.googleapis.com",
                    "methodName": "beta.compute.instances.insert",
                    "resourceName": "projects/western-verve-123456/zones/asia-northeast1-b/instances/instance-5",
                    "request": {"@type": "type.googleapis.com/compute.instances.insert"},
                },
                "insertId": "-5tqx5fd4mj8",
                "resource": {
                    "type": "gce_instance",
                    "labels": {
                        "instance_id": "8498166540490993880",
                        "project_id": "western-verve-123456",
                        "zone": "southamerica-east1-b",
                    },
                },
                "timestamp": "2020-05-15T17:15:42.415Z",
                "severity": "NOTICE",
                "logName": "projects/western-verve-123456/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {
                    "id": "operation-1589562934964-5a5b2f61631d6-cc67597a-98092474",
                    "producer": "compute.googleapis.com",
                    "last": True,
                },
                "receiveTimestamp": "2020-05-15T17:15:43.377082868Z",
            },
        ),
        RuleTest(
            name="Create GCS in Asia",
            expected_result=True,
            log={
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "status": {},
                    "authenticationInfo": {"principalEmail": "user.name@runpanther.io"},
                    "requestMetadata": {
                        "callerIp": "136.24.229.58",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe),gzip(gfe)",
                        "requestAttributes": {"time": "2020-05-15T17:25:07.810848781Z", "auth": {}},
                        "destinationAttributes": {},
                    },
                    "serviceName": "storage.googleapis.com",
                    "methodName": "storage.buckets.create",
                    "authorizationInfo": [
                        {
                            "resource": "projects/_/buckets/jacks-test-bucket-200",
                            "permission": "storage.buckets.create",
                            "granted": True,
                            "resourceAttributes": {},
                        },
                    ],
                    "resourceName": "projects/_/buckets/jacks-test-bucket-200",
                    "serviceData": {
                        "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                        "policyDelta": {
                            "bindingDeltas": [
                                {
                                    "action": "ADD",
                                    "role": "roles/storage.legacyBucketOwner",
                                    "member": "projectEditor:western-verve-123456",
                                },
                                {
                                    "action": "ADD",
                                    "role": "roles/storage.legacyBucketOwner",
                                    "member": "projectOwner:western-verve-123456",
                                },
                                {
                                    "action": "ADD",
                                    "role": "roles/storage.legacyBucketReader",
                                    "member": "projectViewer:western-verve-123456",
                                },
                            ],
                        },
                    },
                    "request": {
                        "defaultObjectAcl": {
                            "bindings": [
                                {
                                    "role": "roles/storage.legacyObjectReader",
                                    "members": ["projectViewer:western-verve-123456"],
                                },
                                {
                                    "members": [
                                        "projectOwner:western-verve-123456",
                                        "projectEditor:western-verve-123456",
                                    ],
                                    "role": "roles/storage.legacyObjectOwner",
                                },
                            ],
                            "@type": "type.googleapis.com/google.iam.v1.Policy",
                        },
                    },
                    "resourceLocation": {"currentLocations": ["asia-northeast2"]},
                },
                "insertId": "c7rgc9c178",
                "resource": {
                    "type": "gcs_bucket",
                    "labels": {
                        "bucket_name": "jacks-test-bucket-200",
                        "project_id": "western-verve-123456",
                        "location": "asia-northeast2",
                    },
                },
                "timestamp": "2020-05-15T17:25:07.807169539Z",
                "severity": "NOTICE",
                "logName": "projects/western-verve-123456/logs/cloudaudit.googleapis.com%2Factivity",
                "receiveTimestamp": "2020-05-15T17:25:09.393448555Z",
            },
        ),
        RuleTest(
            name="BigQuery access log (does not have standard attribute: resource.labels.location)",
            expected_result=False,
            log={
                "insertId": "v3a96bedw1us",
                "logName": "projects/western-verve-123456/logs/cloudaudit.googleapis.com%2Fdata_access",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user.name@runpanther.io"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "bigquery.jobs.create",
                            "resource": "projects/western-verve-123456",
                            "resourceAttributes": {},
                        },
                    ],
                    "methodName": "jobservice.insert",
                    "requestMetadata": {
                        "callerIP": "192.0.2.0",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:92.0) Gecko/20100101 Firefox/92.0,gzip(gfe),gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {},
                    },
                    "resourceName": "projects/western-verve-123456/jobs",
                    "serviceData": {
                        "@type": "type.googleapis.com/google.cloud.bigquery.logging.v1.AuditData",
                        "jobInsertRequest": {
                            "resource": {
                                "jobConfiguration": {
                                    "dryRun": True,
                                    "query": {
                                        "createDisposition": "CREATE_IF_NEEDED",
                                        "defaultDataset": {},
                                        "destinationTable": {},
                                        "query": "select * from no_such_table",
                                        "queryPriority": "QUERY_INTERACTIVE",
                                        "writeDisposition": "WRITE_EMPTY",
                                    },
                                },
                                "jobName": {"location": "US", "projectId": "western-verve-123456"},
                            },
                        },
                        "jobInsertResponse": {
                            "resource": {
                                "jobConfiguration": {},
                                "jobName": {},
                                "jobStatistics": {},
                                "jobStatus": {"error": {}, "state": "PENDING"},
                            },
                        },
                    },
                    "serviceName": "bigquery.googleapis.com",
                    "status": {"code": 11, "message": "Syntax error: Unexpected end of script at [17:7]"},
                },
                "receiveTimestamp": "2021-10-19 16:09:26.351861539",
                "resource": {"labels": {"project_id": "western-verve-123456"}, "type": "bigquery_resource"},
                "severity": "ERROR",
                "timestamp": "2021-10-19 16:09:26.013200000",
            },
        ),
    ]



import json

from policyuniverse.policy import Policy

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSIAMBackdoorRole(Rule):
    id = "AWS.IAM.BackdoorRole-prototype"
    display_name = "IAM Role Policy Updated to Allow Internet Access"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Security Control", "IAM"]
    reports = {"CIS": ["1.1"], "MITRE ATT&CK": ["TA0007:T1078"]}
    default_severity = Severity.MEDIUM
    default_description = "An IAM role policy was updated to allow internet access, which could indicate a backdoor.\n"
    default_runbook = "Check if the action was authorized and if the policy was updated by a trusted user. If not, revert the policy and investigate the user"
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html"

    def rule(self, event):
        if not aws_cloudtrail_success(event) or event.get("eventName") != "UpdateAssumeRolePolicy":
            return False
        policy = event.deep_get("requestParameters", "policyDocument", default="{}")
        return Policy(json.loads(policy)).is_internet_accessible()

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="IAM Role Policy Updated to Allow Internet Access",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "UpdateAssumeRolePolicy",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {
                    "policyDocument": '{"Version":"2012-10-17","Statement":[{"Effect":"Allow","Principal":"*","Action":"sts:AssumeRole","Condition":{"StringEquals":{"sts:ExternalId":"12345"}}}]}',
                },
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="IAM Role Policy Updated Without Internet Access",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "UpdateAssumeRolePolicy",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {
                    "policyDocument": '{"Version":"2012-10-17","Statement":[{"Effect":"Allow","Principal":{"Service":"ec2.amazonaws.com"},"Action":"sts:AssumeRole"}]}',
                },
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="IAM Role Policy Updated With No Policy Document",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "UpdateAssumeRolePolicy",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
    ]



from pypanther import LogType, Rule, Severity, panther_managed


@panther_managed
class AWSConsoleLogin(Rule):
    id = "AWS.Console.Login-prototype"
    display_name = "AWS Console Login"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    create_alert = False

    def rule(self, event):
        return event.get("eventName") == "ConsoleLogin"

    def alert_context(self, event):
        context = {}
        context["ip_and_username"] = event.get("sourceIPAddress", "<MISSING_SOURCE_IP>") + event.deep_get(
            "userIdentity",
            "userName",
            default="<MISSING_USER_NAME>",
        )
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class SnowflakeStreamLoginSuccess(Rule):
    id = "Snowflake.Stream.LoginSuccess-prototype"
    display_name = "Snowflake Successful Login"
    log_types = [LogType.SNOWFLAKE_LOGIN_HISTORY]
    default_severity = Severity.INFO
    create_alert = False
    default_description = "Track successful login signals for correlation."
    tags = ["Snowflake"]

    def rule(self, event):
        return all((event.get("EVENT_TYPE") == "LOGIN", event.get("IS_SUCCESS") == "YES"))

    tests = [
        RuleTest(
            name="Successful Login",
            expected_result=True,
            log={
                "p_event_time": "2024-10-08 14:38:46.061000000",
                "p_log_type": "Snowflake.LoginHistory",
                "p_source_label": "Snowflake Prod",
                "CLIENT_IP": "1.1.1.1",
                "EVENT_ID": "393754014361778",
                "EVENT_TIMESTAMP": "2024-10-08 14:38:46.061000000",
                "EVENT_TYPE": "LOGIN",
                "FIRST_AUTHENTICATION_FACTOR": "PASSWORD",
                "IS_SUCCESS": "YES",
                "RELATED_EVENT_ID": "0",
                "REPORTED_CLIENT_TYPE": "OTHER",
                "REPORTED_CLIENT_VERSION": "1.11.1",
                "USER_NAME": "ckent@dailyplanet.org",
            },
        ),
        RuleTest(
            name="Unsuccessful Login",
            expected_result=False,
            log={
                "p_event_time": "2024-10-08 14:38:46.061000000",
                "p_log_type": "Snowflake.LoginHistory",
                "p_source_label": "Snowflake Prod",
                "CLIENT_IP": "1.2.3.4",
                "EVENT_ID": "393754014361778",
                "EVENT_TIMESTAMP": "2024-10-08 14:38:46.061000000",
                "EVENT_TYPE": "LOGIN",
                "FIRST_AUTHENTICATION_FACTOR": "PASSWORD",
                "IS_SUCCESS": "NO",
                "RELATED_EVENT_ID": "0",
                "REPORTED_CLIENT_TYPE": "OTHER",
                "REPORTED_CLIENT_VERSION": "1.11.1",
                "USER_NAME": "luthor@lexcorp.com",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.slack import slack_alert_context


@panther_managed
class SlackAuditLogsPrivateChannelMadePublic(Rule):
    id = "Slack.AuditLogs.PrivateChannelMadePublic-prototype"
    display_name = "Slack Private Channel Made Public"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = [
        "Slack",
        "Defense Evasion",
        "File and Directory Permissions Modification",
        "Persistence",
        "Account Manipulation",
        "Exfiltration",
        "Exfiltration Over Web Service",
    ]
    reports = {"MITRE ATT&CK": ["TA0005:T1222", "TA0003:T1098", "TA0010:T1567"]}
    default_severity = Severity.HIGH
    default_description = "Detects when a channel that was previously private is made public"
    default_reference = "https://slack.com/intl/en-gb/help/articles/213185467-Convert-a-channel-to-private-or-public"
    summary_attributes = ["p_any_ip_addresses", "p_any_emails"]

    def rule(self, event):
        return event.get("action") == "private_channel_converted_to_public"

    def alert_context(self, event):
        return slack_alert_context(event)

    tests = [
        RuleTest(
            name="Private Channel Made Public",
            expected_result=True,
            log={
                "action": "private_channel_converted_to_public",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "A012B3CDEFG",
                        "name": "username",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace",
                        "id": "T01234N56GB",
                        "name": "test-workspace",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
            },
        ),
        RuleTest(
            name="User Logout",
            expected_result=False,
            log={
                "action": "user_logout",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace-1",
                        "id": "T01234N56GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
                "date_create": "2022-07-28 15:22:32",
                "entity": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GCPBigQueryLargeScan(Rule):
    default_description = "Detect any BigQuery query that is doing a very large scan (> 1 GB)."
    display_name = "GCP BigQuery Large Scan"
    default_reference = "https://cloud.google.com/bigquery/docs/running-queries"
    default_severity = Severity.INFO
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.BigQuery.Large.Scan-prototype"
    # 1.07 GB
    QUERY_THRESHOLD_BYTES = 1073741824

    def rule(self, event):
        return all(
            [
                event.deep_get("resource", "type", default="<type not found>").startswith("bigquery"),
                event.deep_get("operation", "last") is True,
                event.deep_get("protoPayload", "metadata", "jobChange", "job", "jobConfig", "type") == "QUERY",
                event.deep_get(
                    "protoPayload",
                    "metadata",
                    "jobChange",
                    "job",
                    "jobConfig",
                    "queryConfig",
                    "statementType",
                )
                == "SELECT",
                int(
                    event.deep_get(
                        "protoPayload",
                        "metadata",
                        "jobChange",
                        "job",
                        "jobStats",
                        "queryStats",
                        "totalBilledBytes",
                        default=0,
                    ),
                )
                > self.QUERY_THRESHOLD_BYTES,
            ],
        )

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        query_size = event.deep_get(
            "protoPayload",
            "metadata",
            "jobChange",
            "job",
            "jobStats",
            "queryStats",
            "totalBilledBytes",
            default=0,
        )
        return f"GCP: [{actor}] ran a large BigQuery query of [{query_size}] bytes."

    def alert_context(self, event):
        return {
            "query": event.deep_get(
                "protoPayload",
                "metadata",
                "jobChange",
                "job",
                "jobConfig",
                "queryConfig",
                "query",
                default="<QUERY_NOT_FOUND>",
            ),
            "actor": event.deep_get(
                "protoPayload",
                "authenticationInfo",
                "principalEmail",
                default="<ACTOR_NOT_FOUND>",
            ),
            "query_size": event.deep_get(
                "protoPayload",
                "metadata",
                "jobChange",
                "job",
                "jobStats",
                "queryStats",
                "totalBilledBytes",
                default=0,
            ),
        }

    tests = [
        RuleTest(
            name="small query",
            expected_result=False,
            log={
                "insertid": "ABCDEFGHIJKL",
                "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Fdata_access",
                "operation": {
                    "id": "0123456789012-gcp-project1:abcdefg_hijklmnop_1234567abcd",
                    "last": True,
                    "producer": "bigquery.googleapis.com",
                },
                "p_any_emails": ["user@company.io"],
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-03-28 17:37:02.096",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-03-28 17:38:14.621",
                "p_row_id": "de00d3dcdaeee4b4d5f7fa9d17b4c203",
                "p_schema_version": 0,
                "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
                "p_source_label": "gcp-logsource",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@company.io"},
                    "authorizationInfo": [
                        {"granted": True, "permission": "bigquery.jobs.create", "resource": "projects/gcp-project1"},
                    ],
                    "metadata": {
                        "@type": "type.googleapis.com/google.cloud.audit.BigQueryAuditMetadata",
                        "jobChange": {
                            "after": "DONE",
                            "job": {
                                "jobConfig": {
                                    "queryConfig": {
                                        "createDisposition": "CREATE_IF_NEEDED",
                                        "destinationTable": "projects/gcp-project1/datasets/_c2b49f742788f188022fcec1f1622e7404b40ce5/tables/anondea8e77183dcdf1cf93a47b5003036409a525808207450ab80db17fcc8cdcf53",
                                        "priority": "QUERY_INTERACTIVE",
                                        "query": "-- This query shows a list of the daily top Google Search terms.\nSELECT\n   *\nFROM `bigquery-public-data.google_trends.top_terms`",
                                        "statementType": "SELECT",
                                        "writeDisposition": "WRITE_TRUNCATE",
                                    },
                                    "type": "QUERY",
                                },
                                "jobName": "projects/gcp-project1/jobs/abcdefg_hijklmnop_1234567abcd",
                                "jobStats": {
                                    "createTime": "2023-03-28T17:36:49.087Z",
                                    "endTime": "2023-03-28T17:37:02.092Z",
                                    "queryStats": {
                                        "billingTier": 1,
                                        "outputRowCount": "43683701",
                                        "referencedTables": [
                                            "projects/bigquery-public-data/datasets/google_trends/tables/top_terms",
                                        ],
                                        "totalBilledBytes": "100000",
                                        "totalProcessedBytes": "100000",
                                    },
                                    "startTime": "2023-03-28T17:36:49.224Z",
                                    "totalSlotMs": "259581",
                                },
                                "jobStatus": {"jobState": "DONE"},
                            },
                        },
                    },
                    "methodName": "google.cloud.bigquery.v2.JobService.InsertJob",
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    },
                    "resourceName": "projects/gcp-project1/jobs/abcdefg_hijklmnop_1234567abcd",
                    "serviceName": "bigquery.googleapis.com",
                    "status": {},
                },
                "receivetimestamp": "2023-03-28 17:37:02.114",
                "resource": {"labels": {"location": "US", "project_id": "gcp-project1"}, "type": "bigquery_project"},
                "severity": "INFO",
                "timestamp": "2023-03-28 17:37:02.096",
            },
        ),
        RuleTest(
            name="Large Query",
            expected_result=True,
            log={
                "insertid": "ABCDEFGHIJKL",
                "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Fdata_access",
                "operation": {
                    "id": "0123456789012-gcp-project1:abcdefg_hijklmnop_1234567abcd",
                    "last": True,
                    "producer": "bigquery.googleapis.com",
                },
                "p_any_emails": ["user@company.io"],
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-03-28 17:37:02.096",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-03-28 17:38:14.621",
                "p_row_id": "de00d3dcdaeee4b4d5f7fa9d17b4c203",
                "p_schema_version": 0,
                "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
                "p_source_label": "gcp-logsource",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@company.io"},
                    "authorizationInfo": [
                        {"granted": True, "permission": "bigquery.jobs.create", "resource": "projects/gcp-project1"},
                    ],
                    "metadata": {
                        "@type": "type.googleapis.com/google.cloud.audit.BigQueryAuditMetadata",
                        "jobChange": {
                            "after": "DONE",
                            "job": {
                                "jobConfig": {
                                    "queryConfig": {
                                        "createDisposition": "CREATE_IF_NEEDED",
                                        "destinationTable": "projects/gcp-project1/datasets/_c2b49f742788f188022fcec1f1622e7404b40ce5/tables/anondea8e77183dcdf1cf93a47b5003036409a525808207450ab80db17fcc8cdcf53",
                                        "priority": "QUERY_INTERACTIVE",
                                        "query": "-- This query shows a list of the daily top Google Search terms.\nSELECT\n   *\nFROM `bigquery-public-data.google_trends.top_terms`",
                                        "statementType": "SELECT",
                                        "writeDisposition": "WRITE_TRUNCATE",
                                    },
                                    "type": "QUERY",
                                },
                                "jobName": "projects/gcp-project1/jobs/abcdefg_hijklmnop_1234567abcd",
                                "jobStats": {
                                    "createTime": "2023-03-28T17:36:49.087Z",
                                    "endTime": "2023-03-28T17:37:02.092Z",
                                    "queryStats": {
                                        "billingTier": 1,
                                        "outputRowCount": "43683701",
                                        "referencedTables": [
                                            "projects/bigquery-public-data/datasets/google_trends/tables/top_terms",
                                        ],
                                        "totalBilledBytes": "3097493504",
                                        "totalProcessedBytes": "3096864198",
                                    },
                                    "startTime": "2023-03-28T17:36:49.224Z",
                                    "totalSlotMs": "259581",
                                },
                                "jobStatus": {"jobState": "DONE"},
                            },
                        },
                    },
                    "methodName": "google.cloud.bigquery.v2.JobService.InsertJob",
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    },
                    "resourceName": "projects/gcp-project1/jobs/abcdefg_hijklmnop_1234567abcd",
                    "serviceName": "bigquery.googleapis.com",
                    "status": {},
                },
                "receivetimestamp": "2023-03-28 17:37:02.114",
                "resource": {"labels": {"location": "US", "project_id": "gcp-project1"}, "type": "bigquery_project"},
                "severity": "INFO",
                "timestamp": "2023-03-28 17:37:02.096",
            },
        ),
    ]



from pypanther.rules.onelogin.onelogin_active_login_activity import (
    OneLoginActiveLoginActivity as OneLoginActiveLoginActivity,
)
from pypanther.rules.onelogin.onelogin_high_risk_failed_login import (
    OneLoginHighRiskFailedLogin as OneLoginHighRiskFailedLogin,
)
from pypanther.rules.onelogin.onelogin_login import OneLoginLogin as OneLoginLogin
from pypanther.rules.onelogin.onelogin_password_accessed import OneLoginPasswordAccess as OneLoginPasswordAccess
from pypanther.rules.onelogin.onelogin_password_changed import OneLoginPasswordChanged as OneLoginPasswordChanged
from pypanther.rules.onelogin.onelogin_remove_authentication_factor import (
    OneLoginAuthFactorRemoved as OneLoginAuthFactorRemoved,
)
from pypanther.rules.onelogin.onelogin_threshold_accounts_deleted import (
    OneLoginThresholdAccountsDeleted as OneLoginThresholdAccountsDeleted,
)
from pypanther.rules.onelogin.onelogin_threshold_accounts_modified import (
    OneLoginThresholdAccountsModified as OneLoginThresholdAccountsModified,
)
from pypanther.rules.onelogin.onelogin_unauthorized_access import (
    OneLoginUnauthorizedAccess as OneLoginUnauthorizedAccess,
)
from pypanther.rules.onelogin.onelogin_user_account_locked import OneLoginUserAccountLocked as OneLoginUserAccountLocked
from pypanther.rules.onelogin.onelogin_user_assumed import OneLoginUserAssumption as OneLoginUserAssumption



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSLambdaUpdateFunctionCode(Rule):
    id = "AWS.Lambda.UpdateFunctionCode-prototype"
    display_name = "Lambda Update Function Code"
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0007:T1078"]}
    default_severity = Severity.MEDIUM
    default_description = (
        "Identifies when the code of a Lambda function is updated, which could indicate a potential security risk.\n"
    )
    default_runbook = "Verify the event details and the user that triggered the event. If the event is expected, no action is required. If the event is unexpected, investigate the user and the function that was updated."
    default_reference = "https://stratus-red-team.cloud/attack-techniques/AWS/aws.persistence.lambda-overwrite-code/"

    def rule(self, event):
        return (
            aws_cloudtrail_success(event)
            and event.get("eventSource") == "lambda.amazonaws.com"
            and event.get("eventName").startswith("UpdateFunctionCode")
        )

    def title(self, event):
        lambda_name = event.deep_get("responseElements", "functionName", default="LAMBDA_NAME_NOT_FOUND")
        return f"[AWS.CloudTrail] User [{event.udm('actor_user')}] updated Lambda function code for [{lambda_name}]"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Lambda Update Function Code Event",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "lambda.amazonaws.com",
                "eventName": "UpdateFunctionCode20150331",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"functionName": "my-lambda-function"},
                "responseElements": {"functionName": "my-lambda-function"},
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
                "p_log_type": "AWS.CloudTrail",
            },
        ),
        RuleTest(
            name="Lambda Update Function Code Event v2",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "lambda.amazonaws.com",
                "eventName": "UpdateFunctionCode20150331v2",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"functionName": "my-lambda-function"},
                "responseElements": {"functionName": "my-lambda-function"},
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
                "p_log_type": "AWS.CloudTrail",
            },
        ),
        RuleTest(
            name="Delete Function Event",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "lambda.amazonaws.com",
                "eventName": "DeleteFunction",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"functionName": "my-lambda-function"},
                "responseElements": {"functionName": "my-lambda-function"},
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
                "p_log_type": "AWS.CloudTrail",
            },
        ),
    ]
